-------------------------------------------------------------------------------
-- Name: class AutoLogoutTimeout
-- Author: Jonathan Primeau
--
-- Description:
--     Takes care of the automatic logout screen.
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- P R O P E R T I E S
------------------------------------------------------------------------------- 

property ancestor              -- Parent
property pWindow               -- Window object


-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = script("class Timeout").new()
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  me.pWindow.CloseWindow()
  me.pWindow = void
  
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  me.ancestor = void
end

on Start (me)
  GenLog(#debug, "AutoLogoutTimeout::Start: Starting timer...")
  
  me.ResetHit()
  me.SetName("to,autoLogout")
  me.SetHitScale(1000) -- 1 second
  
  lTimeout = integer(QI(#config).GetTimeout())
  me.SetTimeout(lTimeout)
  
  me.SetHitCallback(#_HitCallback, me)
  me.SetTimeoutCallback(#_TimeoutCallback, me)
  
  if (ancestor.Start() = true) then
    GenLog(#debug, "AutoLogoutTimeout::Start: Timer started!")
  else 
    GenLog(#error, "AutoLogoutTimeout::Start: Timer NOT started!")
  end if
end

on Stop (me)
  GenLog(#debug, "AutoLogoutTimeout::Stop: Stopping timer...")
  ancestor.Stop()
  GenLog(#debug, "AutoLogoutTimeout::Stop: Timer stopped!")
end

on Show (me)
  -- Make sure that we are logged in before showing this
  if (QI(#patron).GetPatronInfo().id <> 0) then
    if (me.pWindow.IsVisible() = false) then
      GenLog(#debug, "AutoLogoutTimeout::Show: Showing the timeout...")
      
      me.SetCountDown("")
      
      me.pWindow.ShowWindow()
      
      me.Start()
      
      me.SetCountDown(integer(QI(#config).GetTimeout()) / 1000)
    end if
  end if
end

on Hide (me, theLogout)
  me.SetCountDown("")
  me.Stop()
  GenLog(#debug, "AutoLogoutTimeout::Hide: Hiding the timeout...")
  
  me.pWindow.HideWindow()
  
  if (theLogout = false) then
    -- Reset the login tinmeout hit
    QI(#appTimeout).ResetHit()
  else
    ActLog(#patron, "Timeout", "Patron timed out")
    QI(#login).Logout()
  end if
end

on SetCountDown (me, theValue)
  me.pWindow.GetMovieRef().member("timeout,countdown").text = string(theValue)
end

-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  me.pWindow = script("class Window").new()
  member("timeout,countdown").text = ""
  
  me.pWindow.OpenWindow("Timeout", #dialog)
end

on _HitCallback (me)
  me.SetCountDown(me.GetCountDownSec())
end

on _TimeoutCallback (me)
  GenLog(#debug, "AutoLogoutTimeout::_TimeoutCallback: Logout...")
  me.Hide()
  QI(#login).LogoutNoEnterbet()
  GenLog(#debug, "AutoLogoutTimeout::_TimeoutCallback: Logout!")
end