-------------------------------------------------------------------------------
-- Name: class Banner
-- Author: Jonathan Primeau
--
-- Description:
--     This class is meant to be a manager for any kind of banner. It manages
--     banner cycling and it is able to display many kind of banners.
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- P R O P E R T I E S
-------------------------------------------------------------------------------

property ancestor              -- Parent
property pBannerList           -- The list of banners to be displayed
property pBannerIndex          -- The current next index

property pBannerType           -- Current banner type
property pBannerPath
property pIsCycling
property pForceCycling
property pBannerXML


-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = script("class XMLParser").new()
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  me.ancestor = void
end

-------------------------------------------------------------------------------
-- void StartCycling (me)
--
-- This method is used to start the timeout for cycling the banners.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on StartCycling (me)
  if (me.pIsCycling = false) then
    
    -- First time, force cycling
    if (me.pForceCycling = true) then
      GenLog(#debug, "Banner::StartCycling: Force cycling...")
      me.Cycle()
      me.pForceCycling = false
    end if
    
    GenLog(#debug, "Banner::StartCycling: Start cycling...")
    QI(#bannerCycleTimeout).Start()
    
    me.pIsCycling = true
    GenLog(#debug, "Banner::StartCycling: Startup done!")
  else
    GenLog(#debug, "Banner::StartCycling: Already cycling!")
  end if
end

-------------------------------------------------------------------------------
-- void Cycle (me)
--
-- This method is called by the banner cycling timeout callback (global).
-- everytime it needs to cycle the banners. It takes care of loading the next
-- banner in the cycling loop.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Cycle (me)
  GenLog(#debug, "Banner::Cycle: Cycling...")
  
  banner = ""
  me._GoToNextBanner()
  
  if (me.pBannerIndex <= me.pBannerList.count) then
    banner = me.pBannerList[me.pBannerIndex]
  end if
  
  GenLog(#debug, "Banner::Cycle: Loading banner:"&&string(banner))
  
  me.pBannerType = banner.type
  
  -- JP NO ENTERBET
  if (QI(#status).GetStatus().login <> "TSC") then
    -- Protects us from endless loop
    if (me._GetNonCrmBannerCount() < 1) then
      return()
    end if
    -- Check for the next non CRM banner
    if (me.pBannerType = #crm) then
      me.Cycle()
      return()
    end if
  end if
  
  if (banner.type = #crm) then
    -- We got a CRM opportunity
    QI(#crm).Load("banner")
  else 
    ACTLOG(#presentation,"Banner" , me.pBannerList[me.pBannerIndex].action )
    if (banner.type = #flash) then
      lBannerFile = me.pBannerPath & banner.filename
      if (QI(#file).FileExists(lBannerFile) = true) then
        if (SpriteExists("banner")) then
          member("banner,flash").filename = lBannerFile
          sprite("banner").member = member("banner,flash")
        end if
      else
        GenLog(#error, "Banner::Cycle: Banner" && lBannerFile && "doesn't exist!")
        SvrLog("Banner: File doesn't exist:" && lBannerFile)
      end if
    else if (banner.type = #bitmap) then
      lBannerFile = me.pBannerPath & banner.filename
      if (QI(#file).FileExists(lBannerFile) = true) then
        if (SpriteExists("banner")) then
          member("banner,image").filename = lBannerFile
          sprite("banner").member = member("banner,image")
        end if
      else
        GenLog(#error, "Banner::Cycle: Banner" && lBannerFile && "doesn't exist!")
        SvrLog("Banner: File doesn't exist:" && lBannerFile)
      end if
    end if
  end if
  me.CenterSprite()
  
  GenLog(#debug, "Banner::Cycle: Banner loaded!")
end

-------------------------------------------------------------------------------
-- void StopCycling (me)
--
-- This method is called to stop the banner cycling. It kills the timeout.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on StopCycling (me)
  GenLog(#debug, "Banner::StopCycling: Cycling stopped!")
  QI(#bannerCycleTimeout).Stop()
  me.pIsCycling = false
  me.ForceCycling()
end

on SetBannerXML (me, theXML)
  me.pBannerXML = theXML
end

on Load (me)
  GenLog(#debug, "Banner::Load: Loading the banner list...")
  
  me.pBannerList = []
  
  me.pBannerIndex = 0
  me.pBannerType = #none
  
  if (me.pBannerXML = "") then
    GenLog(#error, "Skin::Load: Empty XML!")
    SvrLog("Skin: Received an empty XML")
    return(-1)
  end if
  
  -- Load and parse XML
  me.LoadXMLFromString(me.pBannerXML)
  lStatus = me.Parse(#_BannerXMLCallback)
  if (lStatus <> 0) then
    return(lStatus)
  end if
  
  GenLog(#debug, "Banner::Load: Banner list loaded!")
  return(0)
end

on Reset (me)
  me.pBannerIndex = 1
end


-------------------------------------------------------------------------------
-- void Clear (me)
--
-- This method clears the banner sprite and centers it.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Clear (me)
  member("banner,image").filename = the moviePath & "Skins\" & "empty.png"
  me.StopCycling()
  
  me.pBannerList = []
  
  me.pBannerIndex = 0
  me.pBannerType = #none
  
  me.pIsCycling = false
  me.pForceCycling = true
end

on ForceCycling (me)
  me.pForceCycling = true
end

on DoAction (me)
  lAction = ""
  if (me.pBannerType = #crm) then
    -- CRM_Banner
    lCrmInfo = QI(#crm).GetCrmInfo()
    if (lCrmInfo <> void) then
      if (lCrmInfo.getaProp(#action) <> void and lCrmInfo.getaProp(#offerName) <> void) then
        QI(#opsCom).ConfirmCrmOffer()
        
        lAction = lCrmInfo.action
        lOfferName = lCrmInfo.offerName
        ActLog(#access, "CRM_Banner", lOfferName & "," && lAction)
      end if
    end if
  else
    -- Banner
    lInfo = me.pBannerList[me.pBannerIndex]
    if (lInfo <> void and lInfo <> [:]) then
      lAction = lInfo.action
      lBannerName = lInfo.name
      ActLog(#access, "Banner", lBannerName & "," && lAction)
    end if
  end if
  
  if (lAction <> "") then
    QI(#nav).Navigate(lAction, true)
  end if
end

on CenterSprite (me)
  if (SpriteExists("banner")) then
    sprite("banner").rect = rect(290, 0, 1024, 84)
  end if
end

on HideSprite (me)
  if (SpriteExists("banner")) then
    sprite("banner").rect = rect(290, 0, 292, 2)
  end if
end


-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  me.pBannerList = []
  me.pBannerPath = the moviePath & "Banners\"
  
  me.pBannerIndex = 0
  me.pBannerType = #none
  
  me.pIsCycling = false
  me.pForceCycling = true
  
  me.pBannerXML = ""
end

on _BannerXMLCallback (me, theItemName, theItemValue)
  case (theItemName) of
    #banner:
      -- Get common information
      lName = me.GetItemValue(theItemValue, "name")
      lDescription = me.GetItemValue(theItemValue, "description")
      lFlash = string(me.GetItemValue(theItemValue, "flash"))
      lImage = string(me.GetItemValue(theItemValue, "image"))
      lCRM = string(me.GetItemValue(theItemValue, "crm"))
      lAction = string(me.GetItemValue(theItemValue, "action"))
      
      lFilename = ""
      if (lFlash <> "") then
        lFilename = lFlash
        lType = #flash
      else if (lImage <> "") then
        lFilename = lImage
        lType = #bitmap
      else
        lFilename = void
        lType = #crm
      end if
      
      lCurrentBanner = [:]
      lCurrentBanner.addProp(#name, lName)
      lCurrentBanner.addProp(#type, lType)
      lCurrentBanner.addProp(#filename, lFilename)
      lCurrentBanner.addProp(#action, lAction)
      
      me.pBannerList.add(lCurrentBanner)
  end case
end

on _GoToNextBanner (me)
  me.pBannerIndex = me.pBannerIndex + 1
  if (me.pBannerIndex > me.pBannerList.count) then
    me.pBannerIndex = 1
  end if
end

on _GetNonCrmBannerCount (me)
  lBannerCount = 0
  lCount = me.pBannerList.count
  repeat with i = 1 to lCount
    if (me.pBannerList[i].type <> #crm) then
      lBannerCount = lBannerCount + 1
    end if
  end repeat
  return(lBannerCount)
end
