-------------------------------------------------------------------------------
-- Name class  CRM
-- Author Jonathan Primeau
--
-- Description
--     Takes care of the CRM banner and interstitial.
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- P R O P E R T I E S
------------------------------------------------------------------------------- 

property ancestor              -- Parent
property pCrmType
property pCrmInfo              -- CRM banner information
property pCallback


-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters
--     me (object) this pointer.
--
-- Return
--     me (object) A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = script("class XMLParser").new()
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters
--     me (object) this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  me.ancestor = void
end

on Load (me, theType)
  -- Ask the OPS to retrieve the banner from the CRM
  if (theType = "banner") then
    QI(#opsCom).FillCrmOffer(theType, #_OnCrmBanner, me)
  else if (theType = "interstitial") then
    QI(#opsCom).FillCrmOffer(theType, #_OnCrmInterstitial, me)
  else if (theType = "QuickBets") then
    QI(#opsCom).FillCrmOffer(theType, #_OnCrmQuickBets, me)
  end if
end

on CheckCrmOffer (me, theType, theHandler, theObject)
  me.pCallback.handler = theHandler
  me.pCallback.object = theObject
  
  QI(#opsCom).FillCrmOffer(theType, #_CheckCrmOfferCallback, me)
end

on GetCrmInfo (me)
  return(me.pCrmInfo)
end

on GetCrmType (me)
  return(me.pCrmType)
end

on HideOffer (me)
  if (SpriteExists("interstitial")) then
    sprite("interstitial").rect = rect(78, 224, 79, 225)
  end if
end

on ShowOffer (me)
  if (SpriteExists("interstitial")) then
    sprite("interstitial").rect = rect(78, 224, 942, 534)
  end if
end

on ApplyCrmBanner (me)
  GenLog(#debug, "CRM::ApplyCrmBanner: Applying CRM banner...")
  
  -- Make sure that the type is banner
  me.pCrmType = "banner"
  
  GenLog(#debug, "CRM::ApplyCrmBanner: Getting flash reference...")
  if (SpriteExists("banner")) then
    lFlashRef = sprite("banner").GetVariable("_root", false)
  else
    return(-101)
  end if
  
  GenLog(#debug, "CRM::ApplyCrmBanner: Filling flash...")
  me._FillFlash(lFlashRef)
  GenLog(#debug, "CRM::ApplyCrmBanner: Flash filled!")
  
  QI(#banner).CenterSprite()
  
  GenLog(#debug, "CRM::ApplyCrmBanner: CRM banner applied!")
end

on ApplyCrmInterstitial (me)
  GenLog(#debug, "CRM::ApplyCrmInterstitial: Applying CRM interstitial...")
  
  -- Make sure that the type is interstitial
  me.pCrmType = "interstitial"
  
  GenLog(#debug, "CRM::ApplyCrmInterstitial: Getting flash reference...")
  if (SpriteExists("interstitial")) then
    lFlashRef = sprite("interstitial").GetVariable("_root", false)
  else
    return(-101)
  end if
  
  GenLog(#debug, "CRM::ApplyCrmInterstitial: Filling flash...")
  me._FillFlash(lFlashRef)
  
  me.ShowOffer()
  
  GenLog(#debug, "CRM::ApplyCrmInterstitial: CRM interstitial applied!")
end

on LoadCurrentInterstitial (me)
  me.HideOffer()
  
  if (me.pCrmInfo <> [:]) then
    
    -- Make sure that the type is interstitial
    me.pCrmType = "interstitial"
    
    -- Load the template
    GenLog(#debug, "CRM::LoadCurrentOffer: Loading the flash...")
    lTemplateFile = me._GetTemplatesPath() & me.pCrmInfo.template
    if (SpriteExists("interstitial")) then
      member("banner,flash").filename = lTemplateFile
      sprite("interstitial").member = member("banner,flash")
    end if
  end if
end


-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters
--     me (object) this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  me.pCrmType = ""
  me.pCallback = [#handler: void, #object: void]
  
  me._ClearCrmInfo()
end

on _ClearCrmInfo (me)
  me.pCrmInfo = [:]
  me.pCrmInfo.addProp(#image, [])
  me.pCrmInfo.addProp(#field, [])
  me.pCrmInfo.addProp(#offername, "")
  me.pCrmInfo.addProp(#ident, "")
  me.pCrmInfo.addProp(#template, "")
  me.pCrmInfo.addProp(#action, "")
end

on _CheckCrmOfferCallback (me, theControl, theData)
  
  lOfferAvailable = false
  
  if (theControl.code = "100" or theControl.code = "200") then
    lOfferAvailable = true
    
    -- Prepare CRM banner holder
    me._ClearCrmInfo()
    
    me.pCrmType = "interstitial"
    
    -- Parse the CRM banner
    me.LoadXMLFromString(theData)
    lStatus = me.Parse(#_CrmXMLCallback)
    if (lStatus <> 0) then
      lOfferAvailable = false
    end if
    
    -- Verify the CRM banner
    if (lStatus = 0) then
      lStatus = me._VerifyCrmContent()
      if (lStatus <> 0) then
        lOfferAvailable = false
      end if
    end if
  end if
  
  if (me.pCallback.handler <> void and me.pCallback.object <> void) then
    call(me.pCallback.handler, me.pCallback.object, lOfferAvailable)
  end if
end

on _OnCrmBanner (me, theControl, theData)
  GenLog(#debug, "CRM::_OnCrmBanner: Parsing CRM banner...")
  
  if (theControl.code = "100" or theControl.code = "200") then
    
    -- Prepare CRM banner holder
    me._ClearCrmInfo()
    
    me.pCrmType = "banner"
    
    -- Parse the CRM banner
    me.LoadXMLFromString(theData)
    lStatus = me.Parse(#_CrmXMLCallback)
    if (lStatus <> 0) then
      return(lStatus)
    end if
    
    lCRMaction = me.GetCrmInfo().action
    ACTLOG(#presentation,"CRM_Banner" , lCRMaction)
    
    -- Verify the CRM banner
    lStatus = me._VerifyCrmContent()
    if (lStatus <> 0) then
      return(lStatus)
    end if
    
    -- Apply CRM banner
    QI(#banner).HideSprite()
    
    -- Load the template
    GenLog(#debug, "CRM::_OnCrmBanner: Loading the flash...")
    lTemplateFile = me._GetTemplatesPath() & me.pCrmInfo.template
    if (SpriteExists("banner")) then
      member("banner,flash").filename = lTemplateFile
      sprite("banner").member = member("banner,flash")
    end if
  else
    GenLog(#error, "CRM::_OnCrmBanner: Received an empty CRM banner!")
  end if
end

on _OnCrmInterstitial (me, theControl, theData)
  GenLog(#debug, "CRM::_OnCrmInterstitial: Parsing CRM interstitial...")
  
  if (theControl.code = "100" or theControl.code = "200") then
    
    -- Prepare CRM banner holder
    me._ClearCrmInfo()
    
    me.pCrmType = "interstitial"
    
    -- Parse the CRM banner
    me.LoadXMLFromString(theData)
    lStatus = me.Parse(#_CrmXMLCallback)
    if (lStatus <> 0) then
      return(lStatus)
    end if
    
    -- Verify the CRM banner
    lStatus = me._VerifyCrmContent()
    if (lStatus <> 0) then
      return(lStatus)
    end if
    
    me.HideOffer()
    
    -- Load the template
    GenLog(#debug, "CRM::_OnCrmInterstitial: Loading the flash...")
    lTemplateFile = me._GetTemplatesPath() & me.pCrmInfo.template
    if (SpriteExists("interstitial")) then
      member("banner,flash").filename = lTemplateFile
      sprite("interstitial").member = member("banner,flash")
    end if
  else
    GenLog(#error, "CRM::_OnCrmInterstitial: Received an empty CRM interstital!")
  end if
end

on _OnCrmQuickBets (me, theControl, theData)
  GenLog(#debug, "CRM::_OnCrmQuickBets: Received a CRM QuickBets...")
  
  if (theControl.code = "100" or theControl.code = "200") then
    QI(#terminalCom).Wager(theData)
  else
    GenLog(#error, "CRM::_OnCrmQuickBets: Received an empty CRM QuickBets!")
  end if
end

on _VerifyCrmContent (me)
  GenLog(#debug, "CRM::_VerifyCrmContent: Verifying CRM content...")
  
  -- Check if we got everything
  if (me.pCrmInfo.getaProp(#offername) = void) then
    GenLog(#error, "CRM::_VerifyCrmContent: Missing offername!")
    SvrLog("CRM: Missing the offername")
    return(-201)
  end if
  
  if (me.pCrmInfo.offername = "") then
    GenLog(#error, "CRM::_VerifyCrmContent: Empty offername!")
    SvrLog("CRM: Empty offername")
    return(-202)
  end if
  
  if (me.pCrmInfo.getaProp(#ident) = void) then
    GenLog(#error, "CRM::_VerifyCrmContent: Missing ident!")
    SvrLog("CRM: Missing the ident")
    return(-203)
  end if
  
  if (me.pCrmInfo.ident = "") then
    GenLog(#error, "CRM::_VerifyCrmContent: Empty ident!")
    SvrLog("CRM: Empty ident")
    return(-204)
  end if
  
  if (me.pCrmInfo.getaProp(#template) = void) then
    GenLog(#error, "CRM::_VerifyCrmContent: Missing template!")
    SvrLog("CRM: Missing the template")
    return(-205)
  end if
  
  if (me.pCrmInfo.template = "") then
    GenLog(#error, "CRM::_VerifyCrmContent: Empty template!")
    SvrLog("CRM: Empty template")
    return(-206)
  end if
  
  lTemplateFile = me._GetTemplatesPath() & me.pCrmInfo.template
  if (me._CheckFile(lTemplateFile) = false) then
    return(-207)
  end if
  
  if (me.pCrmInfo.getaProp(#action) = void) then
    GenLog(#error, "CRM::_VerifyCrmContent: Missing action!")
    SvrLog("CRM: Missing the action")
    return(-208)
  end if
  
  if (me.pCrmInfo.action = "") then
    GenLog(#error, "CRM::_VerifyCrmContent: Empty action!")
    SvrLog("CRM: Empty action")
    return(-209)
  end if
  
  -- Check if the images are valid
  lImageCount = me.pCrmInfo.image.count
  repeat with i = 1 to lImageCount
    lName = me.pCrmInfo.image[i].name
    
    if (lName = "") then
      GenLog(#error, "CRM::_VerifyCrmContent: Empty image name!")
      SvrLog("CRM: Empty image name")
      return(-210)
    end if
    
    lFile = me.pCrmInfo.image[i].file
    
    lImageFile = me._GetMediaPath() & lFile
    if (me._CheckFile(lImageFile) = false) then
      return(-211)
    end if
  end repeat
  
  GenLog(#debug, "CRM::_VerifyCrmContent: CRM content OK!")
  return(0)
end

on _FillFlash (me, theFlashRef)
  lImageCount = me.pCrmInfo.image.count
  
  GenLog(#debug, "CRM::_FillFlash: Filling images...")
  if (lImageCount > 0) then
    repeat with i = 1 to lImageCount
      lName = me.pCrmInfo.image[i].name
      lFile = me.pCrmInfo.image[i].file
      lImageFile = QUOTE & me._GetMediaPath() & lFile & QUOTE
      
      if (theFlashRef.getaProp(symbol(lName)) <> void) then
        do "theFlashRef." & lName & ".loadMovie(" & lImageFile & ")"
      else
        GenLog(#error, "CRM::_FillFlash: Missing image item" && lName && "in flash!")
        SvrLog("CRM: Missing image item" && lName && "in flash!")
      end if
    end repeat
  end if
  
  lTextCount = me.pCrmInfo.field.count
  
  GenLog(#debug, "CRM::_FillFlash: Filling texts...")
  if (lTextCount > 0) then
    repeat with i = 1 to lTextCount
      lName = me.pCrmInfo.field[i].name
      lText = me.pCrmInfo.field[i].text
      lText = SearchReplaceString(lText, QUOTE, "&quot;")
      lText = QUOTE & lText & QUOTE
      
      if (theFlashRef.getaProp(symbol(lName)) <> void) then
        do "theFlashRef." & lName & ".htmlText = " & lText
      else
        GenLog(#error, "CRM::_FillFlash: Missing text item" && lName && "in flash!")
        SvrLog("CRM: Missing text item" && lName && "in flash!")
      end if
    end repeat
  end if
end

on _CrmXMLCallback (me, theItemName, theItemValue)
  case (theItemName) of
    #offername:
      lOfferName = string(me.GetPathItemValue(theItemValue, "child.value"))
      me.pCrmInfo.setProp(#offername, lOfferName)
      
    #ident:
      lIdent = string(me.GetPathItemValue(theItemValue, "child.value"))
      me.pCrmInfo.setProp(#ident, lIdent)
      
    #template:
      lTemplate = string(me.GetPathItemValue(theItemValue, "child.value"))
      me.pCrmInfo.setProp(#template, lTemplate)
      
    #action:
      lAction = me.GetPathItemValue(theItemValue, "child.value")
      me.pCrmInfo.setProp(#action, lAction)
      
    #image:
      lName = string(me.GetAttribute(theItemValue, "name"))
      lFile = string(me.GetPathItemValue(theItemValue, "child.value"))
      
      lImage = [:]
      lImage.addProp(#name, lName)
      lImage.addProp(#file, lFile)
      
      me.pCrmInfo.getProp(#image).add(lImage)
      
    #field:
      lName = string(me.GetAttribute(theItemValue, "name"))
      lText = string(me.GetPathItemValue(theItemValue, "child.value"))
      
      lField = [:]
      lField.addProp(#name, lName)
      lField.addProp(#text, lText)
      
      me.pCrmInfo.getProp(#field).add(lField)
  end case
end

on _CheckFile (me, theFile)
  -- Check if the file exists
  if (QI(#file).FileExists(theFile) = false) then
    GenLog(#error, "CRM::_CheckFile: Missing file:" && theFile)
    SvrLog("CRM: Missing file" && theFile)
    return(false)
  end if
  return(true)
end

on _GetMediaPath (me)
  lPath = ""
  if (me.pCrmType = "banner") then
    lPath = the moviePath & "CRM\Banners\Media\"
  else if (me.pCrmType = "interstitial") then
    lPath = the moviePath & "CRM\Interstitial\Media\"
  end if
  return(lPath)
end

on _GetTemplatesPath (me)
  lPath = ""
  if (me.pCrmType = "banner") then
    lPath = the moviePath & "CRM\Banners\Templates\"
  else if (me.pCrmType = "interstitial") then
    lPath = the moviePath & "CRM\Interstitial\Templates\"
  end if
  return(lPath)
end
