-------------------------------------------------------------------------------
-- Name: class Configuration
-- Author: Jonathan Primeau
--
-- Description:
--     This class takes care of the configuration which is loaded from XML.
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- P R O P E R T I E S
-------------------------------------------------------------------------------

property ancestor              -- Parent
property pActivityLogHeartBeat -- Heartbeat Period in seconds
property pBannerPresentationPolicy  
property pAreaTimeout          -- Area timeout information
property pConfigVersion        -- Configuration version
property pScrollingStep
property pVideoSettings
property pOPSInfo              -- Operation Server information
property pOPSSlot
property pCurrentSlot
property pTerminal             -- Terminal Information
property pFoodBeverageURL
property pKioskName
property pKioskVersion
property pOfferTimeout
property pLoginTimeout
property pLogLevel
property pDebugFlag
property pDeniedKeys
property pSecurityPassphrase
property pSecurityActive
property pSecurityLicense
property pSecurityKeepFocus
property pActivityLogPolicy
property pShowDebugInfo
property pShowGeneralInfo
property pAppServerURL
property pSwitchBrowserPolicy
property pClearErrorDelay
property pLowEndUserAgent
property pFatalErrorMessage
property pAccessDeniedMessage
property pWebLoadingMessage
property pSwipeCardURL
property pWinTVSize
property pGSISInfo
property pSCGUrl
property pSciGamesInfo
property pCardReaderInfo

-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  -- We need XML Parser
  me.ancestor = script("class XMLParser").new()
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  me.ancestor = void
end

-------------------------------------------------------------------------------
-- void Load (me)
--
-- This method loads the configuration. You need to call this method to load 
-- the configuration. It parses the configuration XML and loads it. All 
-- attributes are then avaiable via Get methods.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Load (me)
  lConfigurationPath = "Configuration\configuration.xml"
  me.LoadXMLFromFile(lConfigurationPath)
  me.Parse(#_XMLCallback)
end

-------------------------------------------------------------------------------
-- integer GetTimeout (me)
--
-- This method returns the timeout value according to the currently displayed
-- section.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     Returns the timeout value.
--
-------------------------------------------------------------------------------
on GetSectionTimeout (me)
  -- Default hardcoded value
  lTimeout = 300
  
  -- Get the timeout using current section
  lSection = QI(#status).GetStatus().area
  
  lSymSection = #global
  if (lSection <> void) then
    lSymSection = symbol(lSection)
  end if
  
  if (me.pAreaTimeout.getaProp(lSymSection) <> void) then
    -- Get the section timeout in miliseconds
    lTimeout = integer(me.pAreaTimeout.getProp(lSymSection))
  else
    if (me.pAreaTimeout.getaProp(#global) <> void) then
      -- Get the global timeout in miliseconds
      lTimeout = integer(me.pAreaTimeout.getProp(#global))
    end if
  end if
  
  return(lTimeout * 1000) -- in miliseconds
end

on GetTimeout (me)
  -- Default hardcoded value
  lTimeout = 5
  
  if (me.pAreaTimeout.getaProp(#timeout) <> void) then
    -- Get the section timeout in miliseconds
    lTimeout = integer(me.pAreaTimeout.getProp(#timeout))
  end if
  
  return(lTimeout * 1000) -- in miliseconds
end

on GetScrollingStep (me)
  return(me.pScrollingStep)
end

on GetVideoSettings (me)
  return(me.pVideoSettings)
end

on GetOPSInfo (me)
  return(me.pOPSInfo)
end

on GetOPSSlot (me, theState)
  if (theState = #next) then
    
    lCount = me.pOPSSlot.count
    me.pCurrentSlot = me.pCurrentSlot + 1
    if (me.pCurrentSlot > lCount) then
      me.pCurrentSlot = 1
    end if
    -- Return the next slot
    
  else if (theState = #random) then
    lRandom = me.pOPSInfo.random_initial_slot
    
    if (lRandom = 1) then
      -- Return random slot
      lCount = me.pOPSSlot.count
      me.pCurrentSlot = random(lCount)
    else
      -- Return first slot
      me.pCurrentSlot = 1
    end if
    
  else
    me.pCurrentSlot = 1
  end if
  
  return(me.pOPSSlot[me.pCurrentSlot])
end

on GetTerminalInfo (me)
  return(me.pTerminal)
end

on GetShowTerminalStatus (me)
  return(me.pTerminal.show_terminal_status)
end

on GetTerminalErrorMessage (me)
  return(me.pTerminal.terminal_error_msg)
end

on GetFoodBeverageURL (me)
  return(me.pFoodBeverageURL)
end

on GetKioskName (me)
  return(me.pKioskName)
end

on GetBannerPresentationPolicy (me)
  return(me.pBannerPresentationPolicy)
end

on GetOfferTimeout (me)
  if (me.pOfferTimeout = void) then me.pOfferTimeout = 10
  return(me.pOfferTimeout * 1000)
end

on GetLoginTimeout (me)
  if (me.pLoginTimeout = void) then me.pLoginTimeout = 10
  return(me.pLoginTimeout * 1000)
end

on GetLogLevel (me)
  return(me.pLogLevel)
end

on GetDebugFlag (me)
  return(me.pDebugFlag)
end

on GetSecurityActive (me)
  return(me.pSecurityActive)
end

on GetSecurityPassphrase (me)
  return(me.pSecurityPassphrase)
end

on GetSecurityLicense (me)
  return(me.pSecurityLicense)
end

on GetSecurityKeepFocus (me)
  return(me.pSecurityKeepFocus)
end

on GetDeniedKeys (me)
  return(me.pDeniedKeys)
end

on GetKioskVersion (me)
  return(me.pKioskVersion)
end

on GetActivityLogPolicy (me)
  return(me.pActivityLogPolicy)
end

on GetHeartBeat (me)
  return(me.pActivityLogHeartBeat)
end

on SetConfiguration (me, theData)
  me.LoadXMLFromString(theData)
  me.Parse(#_XMLCallback)
end

on GetShowDebugInfo (me)
  return(me.pShowDebugInfo)
end

on GetShowGeneralInfo (me)
  return(me.pShowGeneralInfo)
end

on GetAppServerURL (me)
  return(me.pAppServerURL)
end

on GetSwitchBrowserPolicy (me)
  return(me.pSwitchBrowserPolicy)
end

on GetClearErrorDelay (me)
  return(me.pClearErrorDelay)
end

on GetLowEndUserAgent (me)
  return(me.pLowEndUserAgent)
end

on GetFatalErrorMessage (me)
  return(me.pFatalErrorMessage)
end

on GetAccessDeniedMessage (me)
  return(me.pAccessDeniedMessage)
end

on GetWebLoadingMessage (me)
  return(me.pWebLoadingMessage)
end

on GetSwipeCardURL (me)
  return(me.pSwipeCardURL)
end

on GetWinTVSize (me)
  return(me.pWinTVSize)
end

on GetGSISInfo (me)
  return(me.pGSISInfo)
end

on GetSciGamesInfo (me)
  return(me.pSciGamesInfo)
end

on GetCardReaderInfo (me)
  return(me.pCardReaderInfo)
end

-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  me.pAreaTimeout = [:]
  me.pConfigVersion = ""
  me.pScrollingStep = 0
  me.pFoodBeverageURL = ""
  
  me.pKioskName = "kiosk_000"
  QI(#status).SetKioskName(me.pKioskName)
  
  me.pKioskVersion = "Unknown"
  me.pOfferTimeout = 0
  me.pLoginTimeout = 0
  me.pLogLevel = #debug -- Initial state
  me.pDebugFlag = false
  me.pDeniedKeys = []
  me.pSecurityPassphrase = ""
  me.pSecurityActive = false
  me.pSecurityLicense = ""
  me.pSecurityKeepFocus = false
  me.pActivityLogPolicy = "disk"
  me.pActivityLogHeartBeat = 60
  me.pSwitchBrowserPolicy = "deny"
  me.pClearErrorDelay = 180
  
  me.pShowDebugInfo = 0
  me.pShowGeneralInfo = 0
  
  me.pAppServerURL = "http://localhost:8080"
  
  me.pLowEndUserAgent = ""
  me.pFatalErrorMessage = ""
  me.pAccessDeniedMessage = ""
  me.pWebLoadingMessage = ""
  
  me.pSwipeCardURL = []
  
  me.pOPSSlot = []
  me.pOPSInfo = [:]
  me.pCurrentSlot = 0
  
  me.pWinTVSize = [:]
  me.pGSISInfo = [#url: "", #browser: ""]
  
  --New for SciGames
  me.pSciGamesInfo = [#url: "http://delmar.webbettor.com/servlet/ExternalSessionHandler", #pollPeriod: 60000]
  me.pCardReaderInfo = [#portName: "COM3", #baudRate: 19200]
end

-------------------------------------------------------------------------------
-- void _XMLCallback (me, theItemName, theItemValue)
--
-- Callback used by the XML parser to return data. This method is the brain of
-- the configuration XML because it knows what it has to look for. 
-- It is specified to the XML Parse method and is called everytime it 
-- encounters an XML tag. If you need to retrieve information from a new tag, 
-- you just have to add the symbol corresponding and use the appropriate path. 
-- You can easily retrieve any data you want using GetItemValue from the XML 
-- parser. Then you have to add the retieved information to the appropriate 
-- list.
--
-- Parameters:
--     me (object): this pointer.
--     theItemName (symbol): The current item name being parsed.
--     theItemValue (list): Contains the information for the current item.
--
-------------------------------------------------------------------------------
on _XMLCallback (me, theItemName, theItemValue)
  case (theItemName) of
    #kiosk_name:
      me.pKioskName = string(me.GetPathItemValue(theItemValue, "child.value"))
      QI(#status).SetKioskName(me.pKioskName)
      
    #kiosk_version:
      me.pKioskVersion = string(me.GetPathItemValue(theItemValue, "child.value"))
      
    #activity_log_policy:
      me.pActivityLogPolicy = string(me.GetPathItemValue(theItemValue, "child.value"))
      QI(#actLog).SetPolicy(me.pActivityLogPolicy)
      
    #activity_log_heartbeat:
      me.pActivityLogHeartBeat = integer(me.GetPathItemValue(theItemValue, "child.value"))
      
      if (me.pActivityLogHeartBeat <= 0) then
        me.pActivityLogHeartBeat = 60
      end if
      
    #banner_presentation_policy:
      me.pBannerPresentationPolicy = integer(me.GetPathItemValue(theItemValue, "child.value"))
      
    #debug:
      me.pDebugFlag = integer(me.GetPathItemValue(theItemValue, "child.value"))
      
    #show_debug_info:
      me.pShowDebugInfo = integer(me.GetPathItemValue(theItemValue, "child.value"))
      
    #show_general_info:
      me.pShowGeneralInfo = integer(me.GetPathItemValue(theItemValue, "child.value"))
      
    #log_level:
      lLogLevel = symbol(me.GetPathItemValue(theItemValue, "child.value"))
      
      me.pLogLevel = lLogLevel
      
    #app_server_url:
      me.pAppServerURL = string(me.GetPathItemValue(theItemValue, "child.value"))
      
    #switch_browser_policy:
      me.pSwitchBrowserPolicy = string(me.GetPathItemValue(theItemValue, "child.value"))
      
    #clear_error_delay:
      me.pClearErrorDelay = integer(me.GetPathItemValue(theItemValue, "child.value"))
      
    #low_end_user_agent:
      me.pLowEndUserAgent = string(me.GetPathItemValue(theItemValue, "child.value"))
      
    #fatal_error_message:
      me.pFatalErrorMessage = string(me.GetPathItemValue(theItemValue, "child.value"))
      
    #access_denied_message:
      me.pAccessDeniedMessage = string(me.GetPathItemValue(theItemValue, "child.value"))
      
    #web_loading_message:
      me.pWebLoadingMessage = string(me.GetPathItemValue(theItemValue, "child.value"))
      
    #ops:
      lWaitRetry = integer(me.GetItemValue(theItemValue, "wait_retry"))
      lNumberRetry = integer(me.GetItemValue(theItemValue, "number_retry"))
      lRandomInitialSlot = integer(me.GetItemValue(theItemValue, "random_initial_slot"))
      
      me.pOPSInfo = [:]
      me.pOPSInfo.addProp(#wait_retry, lWaitRetry)
      me.pOPSInfo.addProp(#number_retry, lNumberRetry)
      if (lRandomInitialSlot = void) then lRandomInitialSlot = 0
      me.pOPSInfo.addProp(#random_initial_slot, lRandomInitialSlot)
      
    #ops_slot:
      lIP = string(me.GetItemValue(theItemValue, "ip"))
      lPort = integer(me.GetItemValue(theItemValue, "port"))
      lCurrentSlot = [:]
      lCurrentSlot.addProp(#ip, lIP)
      lCurrentSlot.addProp(#port, lPort)
      me.pOPSSlot.add(lCurrentSlot)
      
    #terminal:
      lIP = string(me.GetItemValue(theItemValue, "ip"))
      lPort = integer(me.GetItemValue(theItemValue, "port"))
      lRetry = integer(me.GetItemValue(theItemValue, "retry"))
      lShowTerminalStatus = integer(me.GetItemValue(theItemValue, "show_terminal_status"))
      lTerminalErrorMsg = string(me.GetItemValue(theItemValue, "terminal_status_error_msg"))
      
      me.pTerminal = [:]
      me.pTerminal.addProp(#ip, lIP)
      me.pTerminal.addProp(#port, lPort)
      me.pTerminal.addProp(#retry, lRetry)
      me.pTerminal.addProp(#show_terminal_status, lShowTerminalStatus)
      me.pTerminal.addProp(#terminal_error_msg, lTerminalErrorMsg)
      
    #web:
      me.pScrollingStep = integer(me.GetItemValue(theItemValue, "scrolling"))
      
    #food_beverage:
      me.pFoodBeverageURL = me.GetItemValue(theItemValue, "url")
      
    #swipe_card_url:
      lSwipeCardURL = string(me.GetPathItemValue(theItemValue, "child.value"))
      me.pSwipeCardURL.add(lSwipeCardURL)
      
    #configuration:
      me.pConfigVersion = me.GetAttribute(theItemValue, "version")
      
    #offer:
      lOfferTimeout = me.GetItemValue(theItemValue, "timeout")
      me.pOfferTimeout = lOfferTimeout
      
    #login:
      lLoginTimeout = me.GetItemValue(theItemValue, "timeout")
      me.pLoginTimeout = lLoginTimeout
      
    #global:
      lGlobalTimeout = me.GetItemValue(theItemValue, "timeout")
      me.pAreaTimeout.addProp(#global, lGlobalTimeout)
      
    #area_config:
      lAreaName = me.GetAttribute(theItemValue, "area")
      lAreaValue = me.GetItemValue(theItemValue, "timeout")
      me.pAreaTimeout.addProp(symbol(lAreaName), lAreaValue)
      
    #video:
      lDefaultChannel = me.GetItemValue(theItemValue, "default_channel")
      lTuneCountry = me.GetItemValue(theItemValue, "tune_country")
      lToolbarPosition = me.GetItemValue(theItemValue, "toolbar_position")
      lContextMenu = me.GetItemValue(theItemValue, "context_menu")
      lTuneSource = me.GetItemValue(theItemValue, "tune_source")
      lContrast = me.GetItemValue(theItemValue, "contrast")
      lBrightness = me.GetItemValue(theItemValue, "brightness")
      lHue = me.GetItemValue(theItemValue, "hue")
      lSaturation = me.GetItemValue(theItemValue, "saturation")
      
      me.pVideoSettings = [:]
      me.pVideoSettings.addProp(#defaultChannel, lDefaultChannel)
      me.pVideoSettings.addProp(#tuneCountry, lTuneCountry)
      me.pVideoSettings.addProp(#toolbarPosition, lToolbarPosition)
      me.pVideoSettings.addProp(#contextMenu, lContextMenu)
      me.pVideoSettings.addProp(#tuneSource, lTuneSource)
      me.pVideoSettings.addProp(#contrast, lContrast)
      me.pVideoSettings.addProp(#brightness, lBrightness)
      me.pVideoSettings.addProp(#hue, lHue)
      me.pVideoSettings.addProp(#saturation, lSaturation)
      
    #security:
      lPassphrase = me.GetItemValue(theItemValue, "passphrase")
      lActive = integer(me.GetItemValue(theItemValue, "active"))
      lLicense = me.GetItemValue(theItemValue, "license")
      lKeepFocus = integer(me.GetItemValue(theItemValue, "keep_focus"))
      
      me.pSecurityPassphrase = lPassphrase
      me.pSecurityActive = lActive
      me.pSecurityLicense = lLicense
      me.pSecurityKeepFocus = lKeepFocus
      
    #denied_key:
      lKey = string(me.GetPathItemValue(theItemValue, "child.value"))
      
      me.pDeniedKeys.add(lKey)
      
    #video_position:
      lName = string(me.GetAttribute(theItemValue, "name"))
      lLeft = integer(me.GetItemValue(theItemValue, "left"))
      lTop = integer(me.GetItemValue(theItemValue, "top"))
      lRight = integer(me.GetItemValue(theItemValue, "right"))
      lBottom = integer(me.GetItemValue(theItemValue, "bottom"))
      
      lRect = rect(lLeft, lTop, lRight, lBottom)
      me.pWinTVSize.addProp(symbol(lName), lRect)
      
    #gsis_url:
      me.pGSISInfo.url = string(me.GetPathItemValue(theItemValue, "child.value"))
      
    #gsis_browser:
      me.pGSISInfo.browser = string(me.GetPathItemValue(theItemValue, "child.value"))
      
    #sci_games:
      lUrl = me.GetItemValue(theItemValue, "url")
      lPollPeriod = 1000 * integer(me.GetItemValue(theItemValue, "pollPeriod"))
      lLoginTimeout = 1000 * integer(me.GetItemValue(theItemValue, "loginTimeout"))
      lWagerOnThisEventUrl = me.GetItemValue(theItemValue, "wagerOnThisEventUrl")
      
      me.pSciGamesInfo = [#url: lUrl, #pollPeriod: lPollPeriod, #loginTimeout: lLoginTimeout, #wagerOnThisEventUrl:lWagerOnThisEventUrl]
      
    #card_reader:
      lportName = me.GetItemValue(theItemValue, "portName")
      lbaudRate = integer(me.GetItemValue(theItemValue, "baudRate"))
      lstartOffset = integer(me.GetItemValue(theItemValue, "startOffset"))
      lendOffset = integer(me.GetItemValue(theItemValue, "endOffset"))
      lbufTimeout = integer(1000 * float(me.GetItemValue(theItemValue, "bufTimeout")))
      
      me.pCardReaderInfo = [#portName: lportName, #baudRate: lbaudRate, #startOffset: lstartOffset, #endOffset: lendOffset, #bufTimeout: lbufTimeout]
      
  end case
end
