-------------------------------------------------------------------------------
-- Name: class DynamicScreen
-- Author: Jonathan Primeau
--
-- Description:
--     This class is the higher level of dynamic screens.
--
------------------------------------------------------------------------------- 

-------------------------------------------------------------------------------
-- P R O P E R T I E S
-------------------------------------------------------------------------------

property ancestor              -- Parent
property pMainButtons
property pSubButtons
property pMaxMainButton
property pCurrentPage
property pItemPerPage
property pMainSpritePrefix
property pSubSpritePrefix
property pOriginalSpriteLoc


-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = void
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  me.ancestor = void
end

on Load (me)
  me.Clear()
  
  me._StoreOriginalSpriteLoc()
  
  me._DisplayMainButtons()
  me._DisplaySubButtons()
end

on Clear (me)
end

on GoNextPage (me)
  if (me._IsNextPageAvailable()) then
    me.pCurrentPage = me.pCurrentPage + 1
    GenLog(#debug, "DynamicScreen::GoNextPage:" && string(me.pCurrentPage))
    me._DisplaySubButtons()
  end if
end

on GoPreviousPage (me)
  if (me._IsPreviousPageAvailable()) then
    me.pCurrentPage = me.pCurrentPage - 1
    GenLog(#debug, "DynamicScreen::GoPreviousPage:" && string(me.pCurrentPage))
    me._DisplaySubButtons()
  end if
end

on DoAction (me, theSpriteName)
  lOldDelimiter = _player.itemDelimiter
  _player.itemDelimiter = ","
  
  lButtonType = theSpriteName.item[2]
  lButtonIndex = integer(theSpriteName.item[3])
  
  lStartIndex = me.pCurrentPage * me.pItemPerPage - (me.pItemPerPage - 1)
  lIndex = lStartIndex + lButtonIndex - 1
  
  lAction = [:]
  if (lButtonType = "main") then
    lAction = me.pMainButtons[lButtonIndex].action
  else if (lButtonType = "sub") then
    lAction = me.pSubButtons[lIndex].action
  end if
  
  GenLog(#debug, "DynamicScreen::DoAction:" && string(lAction))
  if (lAction <> "") then
    QI(#nav).Navigate(lAction, true)
  end if
  
  _player.itemDelimiter = lOldDelimiter
end


-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  me.pMainButtons = []
  me.pSubButtons = []
  me.pOriginalSpriteLoc = []
  
  me.pCurrentPage = 1
end

on _StoreOriginalSpriteLoc (me)
  -- Store the main buttons loc
  repeat with i = 1 to me.pMaxMainButton
    lSpriteName = me.pMainSpritePrefix & i
    lSpriteLabelName = me.pMainSpritePrefix & "label," & i
    
    -- Graphic
    lSprite = [:]
    lSprite.addProp(#name, sprite(lSpriteName).name)
    lSprite.addProp(#loc, sprite(lSpriteName).loc)
    me.pOriginalSpriteLoc.add(lSprite)
    
    -- Label
    lSprite = [:]
    lSprite.addProp(#name, sprite(lSpriteLabelName).name)
    lSprite.addProp(#loc, sprite(lSpriteLabelName).loc)
    me.pOriginalSpriteLoc.add(lSprite)
  end repeat
  
  -- Store the sub buttons loc
  repeat with i = 1 to me.pItemPerPage
    lSpriteName = me.pSubSpritePrefix & i
    lSpriteLabelName = me.pSubSpritePrefix & "label," & i
    
    -- Graphic
    lSprite = [:]
    lSprite.addProp(#name, sprite(lSpriteName).name)
    lSprite.addProp(#loc, sprite(lSpriteName).loc)
    me.pOriginalSpriteLoc.add(lSprite)
    
    -- Label
    lSprite = [:]
    lSprite.addProp(#name, sprite(lSpriteLabelName).name)
    lSprite.addProp(#loc, sprite(lSpriteLabelName).loc)
    me.pOriginalSpriteLoc.add(lSprite)
  end repeat
  
  -- Store the container loc
  lSprite = [:]
  lSprite.addProp(#name, "container")
  lSprite.addProp(#loc, sprite("container").loc)
  me.pOriginalSpriteLoc.add(lSprite)
  
  -- Store the page up (graphic)
  lSprite = [:]
  lSprite.addProp(#name, "pageUp")
  lSprite.addProp(#loc, sprite("pageUp").loc)
  me.pOriginalSpriteLoc.add(lSprite)
  
  -- Store the page up (label)
  lSprite = [:]
  lSprite.addProp(#name, "pageUp,text")
  lSprite.addProp(#loc, sprite("pageUp,text").loc)
  me.pOriginalSpriteLoc.add(lSprite)
  
  -- Store the page down (graphic)
  lSprite = [:]
  lSprite.addProp(#name, "pageDown")
  lSprite.addProp(#loc, sprite("pageDown").loc)
  me.pOriginalSpriteLoc.add(lSprite)
  
  -- Store the page down (label)
  lSprite = [:]
  lSprite.addProp(#name, "pageDown,text")
  lSprite.addProp(#loc, sprite("pageDown,text").loc)
  me.pOriginalSpriteLoc.add(lSprite)
  
  -- Store the page information
  lSprite = [:]
  lSprite.addProp(#name, "pageInfo")
  lSprite.addProp(#loc, sprite("pageInfo").loc)
  me.pOriginalSpriteLoc.add(lSprite)
end

on _GetHiddenLoc (me)
  return(point(-500, -500))
end

on _GetOriginalLoc (me, theSpriteName)
  lCount = me.pOriginalSpriteLoc.count
  repeat with i = 1 to lCount
    if (me.pOriginalSpriteLoc[i].name = theSpriteName) then
      return(me.pOriginalSpriteLoc[i].loc)
    end if
  end repeat
  return(me._GetHiddenLoc())
end

on _DisplayMainButtons (me)
  GenLog(#debug, "DynamicScreen::_DisplayMainButtons: Displaying main buttons")
  lItemCount = me.pMainButtons.count
  
  repeat with i = 1 to me.pMaxMainButton
    lButtonSprite = me.pMainSpritePrefix & i
    lLabelSprite = me.pMainSpritePrefix & "label," & i
    
    if (i <= lItemCount) then
      lButton = me.pMainButtons[i]
      sprite(lLabelSprite).member.text = string(lButton.text)
      sprite(lButtonSprite).loc = me._GetOriginalLoc(lButtonSprite)
      sprite(lLabelSprite).loc = me._GetOriginalLoc(lLabelSprite)
    else
      sprite(lLabelSprite).member.text = ""
      sprite(lButtonSprite).loc = me._GetHiddenLoc()
      sprite(lLabelSprite).loc = me._GetHiddenLoc()
    end if
    
  end repeat
  GenLog(#debug, "DynamicScreen::_DisplayMainButtons: Main buttons")
end

on _DisplaySubButtons (me)
  GenLog(#debug, "DynamicScreen::_DisplaySubButtons: Displaying sub buttons")
  -- Calculate the starting index
  lStartIndex = me.pCurrentPage * me.pItemPerPage - (me.pItemPerPage - 1)
  
  lItemCount = me.pSubButtons.count
  
  -- Container
  if (lItemCount = 0) then
    -- Hide the container
    sprite("container").loc = me._GetHiddenLoc()
  else
    -- Show the container
    sprite("container").loc = me._GetOriginalLoc("container")
  end if
  
  -- Page Up/Down
  if (me._GetPageCount() <= 1) then
    -- Hide the page up/down
    sprite("pageUp").loc = me._GetHiddenLoc()
    sprite("pageUp,text").loc = me._GetHiddenLoc()
    sprite("pageDown").loc = me._GetHiddenLoc()
    sprite("pageDown,text").loc = me._GetHiddenLoc()
    sprite("pageInfo").loc = me._GetHiddenLoc()
  else
    -- Show the page up/down
    sprite("pageUp").loc = me._GetOriginalLoc("pageUp")
    sprite("pageUp,text").loc = me._GetOriginalLoc("pageUp,text")
    sprite("pageDown").loc = me._GetOriginalLoc("pageDown")
    sprite("pageDown,text").loc = me._GetOriginalLoc("pageDown,text")
    sprite("pageInfo").loc = me._GetOriginalLoc("pageInfo")
  end if
  
  repeat with i = 1 to me.pItemPerPage
    lButtonSprite = me.pSubSpritePrefix & i
    lLabelSprite = me.pSubSpritePrefix & "label," & i
    
    if (lStartIndex <= lItemCount) then
      lButton = me.pSubButtons[lStartIndex]
      sprite(lLabelSprite).member.text = string(lButton.text)
      -- Inserted Color controled from Skin
      sprite(lLabelSprite).member.color = rgb(QI(#skin).GetSecButtonTextColor())
      sprite(lButtonSprite).loc = me._GetOriginalLoc(lButtonSprite)
      sprite(lLabelSprite).loc = me._GetOriginalLoc(lLabelSprite)
    else
      sprite(lLabelSprite).member.text = ""
      sprite(lButtonSprite).loc = me._GetHiddenLoc()
      sprite(lLabelSprite).loc = me._GetHiddenLoc()
    end if
    
    lStartIndex = lStartIndex + 1
  end repeat
  
  sprite("pageInfo").member.text = "Page" && me.pCurrentPage && "of" && me._GetPageCount()
  
  GenLog(#debug, "DynamicScreen::_DisplaySubButtons: Sub buttons")
end

on _IsNextPageAvailable (me)
  lCount = me.pSubButtons.count
  
  lLastIndex = me.pCurrentPage * me.pItemPerPage
  
  if (lCount > lLastIndex) then
    return(true)
  end if
  return(false)
end

on _IsPreviousPageAvailable (me)
  if (me.pCurrentPage > 1) then
    return(true)
  end if
  return(false)
end

on _GetPageCount (me)
  lSubButtonCount = me.pSubButtons.count
  lDivide = me.pItemPerPage
  if (lDivide = 0) then lDivide = 1
  lPageCount = (lSubButtonCount - 1) / lDivide + 1
  return(lPageCount)
end
