-------------------------------------------------------------------------------
-- Name: class History
-- Author: Jonathan Primeau
--
-- Description:
--     This class manages history list. It is used in the interface navigation
--     and also in web browsing
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- P R O P E R T I E S
------------------------------------------------------------------------------- 

property ancestor              -- Parent
property pHistoryList
property pHistoryIndex

-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = void
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  me.ancestor = void
end

on ClearHistory (me)
  me.pHistoryList = []
  me.pHistoryIndex = 0
end

-------------------------------------------------------------------------------
-- void AddToHistory (me)
--
-- Adds navigation to history with its properties.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on AddToHistory (me, theNavProps)
  GenLog(#debug, "History::AddToHistory:" && string(theNavProps))
  -- Delete everything after the index (if any)
  if (listP(me.pHistoryList)) then
    lHistoryCount = me.pHistoryList.count
    lRemoveItemCount = lHistoryCount - me.pHistoryIndex
    repeat with i = 1 to lRemoveItemCount
      lHistoryCount = me.pHistoryList.count
      me.pHistoryList.deleteAt(lHistoryCount)
    end repeat
  end if
  
  -- Add to history
  me.pHistoryList.Add(theNavProps)
  
  -- Adjust the index
  me.pHistoryIndex = me.pHistoryList.count
end

on AddUniqueToHistory (me, theValue)
  if (me.GetLast() <> theValue) then
    me.AddToHistory(theValue)
  end if
end

-------------------------------------------------------------------------------
-- void GoBack (me)
--
-- Goes back in history and updates the current index.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on GoBack (me)
  if (me._IsGoBackAvailable()) then
    me.pHistoryIndex = me.pHistoryIndex - 1
    GenLog(#debug, "History::GoBack:" && string(me.pHistoryIndex))
    lValue = me.pHistoryList[me.pHistoryIndex]
    return(lValue)
  end if
  return(void)
end

-------------------------------------------------------------------------------
-- void GoForward (me)
--
-- Goes forward in history and updates the current index.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on GoForward (me)
  if (me._IsGoForwardAvailable()) then
    me.pHistoryIndex = me.pHistoryIndex + 1
    GenLog(#debug, "History::GoForward:" && string(me.pHistoryIndex))
    lValue = me.pHistoryList[me.pHistoryIndex]
    return(lValue)
  end if
  return(void)
end

on GetLast (me)
  lCount = me.pHistoryList.count
  if (lCount > 0) then
    lValue = me.pHistoryList[lCount]
    return(lValue)
  end if
  return(void)
end

-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  me.ClearHistory()
end


on _IsInHistory (me, theValue)
  if (listP(me.pHistoryList)) then
    lCount = me.pHistoryList.count
    repeat with i = 1 to lCount
      if (theValue = me.pHistoryList[i]) then
        return(true)
      end if
    end repeat
  end if
  return(false)
end

-------------------------------------------------------------------------------
-- bool _IsGoBackAvailable (me)
--
-- Checks if we can go back in the history from current index.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     True if we can go back.
--
-------------------------------------------------------------------------------
on _IsGoBackAvailable (me)
  if (me.pHistoryIndex > 1) then
    return(true)
  end if
  return(false)
end



-------------------------------------------------------------------------------
-- bool _IsGoForwardAvailable (me)
--
-- Checks if we can go foward in the history from current index.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     True if we can go forward.
--
-------------------------------------------------------------------------------
on _IsGoForwardAvailable (me)
  lHistoryCount = 0
  if (listP(me.pHistoryList)) then
    lHistoryCount = me.pHistoryList.count
  end if
  
  if (lHistoryCount = 0) then return(false)
  
  if (me.pHistoryIndex < lHistoryCount) then
    return(true)
  end if
  return(false)
end

