-------------------------------------------------------------------------------
-- Name: class Initialization
-- Author: Jonathan Primeau
--
-- Description:
--     Takes care of kiosk initialization.
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- P R O P E R T I E S
------------------------------------------------------------------------------- 

property ancestor              -- Parent
property pIsInitialized
property pIsLoaded


-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = script("class XMLParser").new()
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  me.ancestor = void
end

on ConnectionTimeout (me, theShowFatalError)
  me.pIsInitialized = false
  me.pIsLoaded = false
  lTarget = "ogn://navigate/initialize"
  QI(#nav).Navigate(lTarget, false)
end

on Reset (me)
  me.pIsInitialized = false
  me.pIsLoaded = false
  
  -- Stop login timeout (if any)
  QI(#appTimeout).Stop()
  
  -- Stop banner cycling (if any)
  QI(#banner).StopCycling()
  
  -- Stop attract loop (if any)
  QI(#attractLoop).StopCycling()
  QI(#AttractSyncTimeout).Stop()
  
  -- Operation Server
  if (QI(#opsCom).IsConnected() = false) then
    if (QI(#opsCom).GetConnectionCount() = 0) then
      -- First connection...
      QI(#opsCom).Connect(#random, #InitializeKiosk, me)
    else
      -- Later connections...
      QI(#opsCom).Connect(#next, #InitializeKiosk, me)
    end if
  else
    me.InitializeKiosk()
  end if
  
  QI(#opsConnectionTimeout).Start()
  
  -- Terminal
  if (QI(#terminalCom).IsConnected() = false) then
    QI(#terminalCom).Connect()
  end if
  
  QI(#terminalConnectionTimeout).Start()
end

on InitializeKiosk (me)
  SetGeneralInfo("Please wait while the kiosk is initializing...")
  QI(#status).ResetStatus()
  QI(#status).SetStatus(#sequence, "Initialization")
  
  QI(#opsCom).InitializeKiosk(#_InitializeKioskCallback, me)
end

on FetchSkinName (me)
  if (QI(#status).GetStatus().admission = "Enabled") then
    QI(#opsCom).FetchSkinName(#_FetchSkinNameCallback, me)
  else
    SetGeneralInfo("Out of service")
  end if
end

on FetchUrlList (me)
  QI(#opsCom).FetchUrlList(#_FetchUrlListCallback, me)
end

on FetchTvChannelList (me)
  QI(#opsCom).FetchTvChannelList(#_FetchTvChannelListCallback, me)
end

on FetchBannerList (me)  
  QI(#opsCom).FetchBannerList(#_FetchBannerListCallback, me)
end

on FetchAttractLoopName (me)  
  QI(#opsCom).FetchAttractLoopName(#_FetchAttractLoopNameCallback, me)
end

on FetchWebProxyList (me)  
  QI(#opsCom).FetchWebProxyList(#_FetchWebProxyListCallback, me)
end

on IsInitialized (me)
  return(me.pIsInitialized)
end

on LoadKiosk (me)
  SetGeneralInfo("Please wait while the kiosk is loading...")
  me.pIsLoaded = false
  
  if (QI(#skin).Load() <> 0) then
    ActLog(#content, "Skin", "ERROR: Bad initial Skin received")
    me._LoadingFailed()
    return()
  else
    ActLog(#content, "Skin", "Initial Skin received")
  end if
  
  if (QI(#internet).Load() <> 0) then
    ActLog(#content, "URL_List", "ERROR: Bad initial URL list received")
    me._LoadingFailed()
    return()
  else
    ActLog(#content, "URL_List", "Initial URL list received")
  end if
  
  if (QI(#video).Load() <> 0) then
    ActLog(#content, "Channel_List", "ERROR: Bad initial channel list received")
    me._LoadingFailed()
    return()
  else
    ActLog(#content, "Channel_List", "Initial channel list received")
  end if
  
  if (QI(#banner).Load() <> 0) then
    ActLog(#content, "Banner_List", "ERROR: Bad initial banner list received")
    me._LoadingFailed()
    return()
  else
    ActLog(#content, "Banner_List", "Initial banner list received")
  end if
  
  if (QI(#attractLoop).Load() <> 0) then
    ActLog(#content, "Attract_Loop", "ERROR: Bad initial attract loop received")
    me._LoadingFailed()
    return()
  else
    ActLog(#content, "Attract_Loop", "Initial attract loop received")
  end if
  
  if (QI(#webProxy).Load() <> 0) then
    ActLog(#content, "WebProxy", "ERROR: Bad initial WebProxy list received")
    me._LoadingFailed()
    return()
  else
    ActLog(#content, "WebProxy", "Initial WebProxy list received")
  end if
  
  --Start Listenining to CardReader
  QI(#CardReader).StartPolling()
  
  me._LoadingCompleted()
end

on IsLoaded (me)
  return(me.pIsLoaded)
end


-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  me.pIsInitialized = false
  me.pIsLoaded = false
end

on _InitializationFailed (me)
  QI(#status).SetStatus(#state, "Critical")
  SetGeneralInfo("Out of service")
end

on _InitializationCompleted (me)
  if (QI(#terminalCom).IsConnected() = true) then
    me.pIsInitialized = true
    QI(#status).SetStatus(#sequence, "Validation")
    QI(#status).SetStatus(#state, "OK")
  else
    me.pIsInitialized = false
    QI(#status).SetStatus(#state, "Critical")
    SetGeneralInfo("Out of service")
  end if
end

on _LoadingFailed (me)
  QI(#status).SetStatus(#state, "Critical")
  SetGeneralInfo("Out of service")
end

on _LoadingCompleted (me)
  me.pIsLoaded = true
  QI(#status).SetStatus(#sequence, "Run")
  QI(#status).SetStatus(#state, "OK")
end

on _InitializeKioskCallback (me, theControl, theData)
  if (listP(theControl)) then
    if (theControl.code = "100" or theControl.code = "200") then
      if (theData <> "") then
        -- Parse InitializeKiosk answer data
        me.LoadXMLFromString(theData)
        me.Parse(#_InitializeKioskXMLCallback)
        
        -- Keep going with initialization
        --me.FetchSkinName()
        
        -- Wait before continue initialization
        QI(#initTimeout).Start()
        
        return()
      end if
    end if
  end if
  
  me._InitializationFailed()
end

on _InitializeKioskXMLCallback (me, theItemName, theItemValue)
  case (theItemName) of
    #info:
      lKioskID = me.GetItemValue(theItemValue, "kiosk_id")
      lKioskName = me.GetItemValue(theItemValue, "name")
      lGroup = me.GetItemValue(theItemValue, "group")
      lLocation = me.GetItemValue(theItemValue, "location")
      lEnabled = value(me.GetItemValue(theItemValue, "enabled"))
      lTime = integer(me.GetItemValue(theItemValue, "time"))
      
      if (lKioskID = void) then
        lKioskID = QI(#config).GetKioskID()
      end if
      QI(#status).SetKioskID(lKioskID)
      QI(#status).SetKioskName(lKioskName)
      QI(#status).SetGroup(lGroup)
      QI(#status).SetLocation(lLocation)
      
      -- Validate and set initialize time
      if (lTime <= 0) then lTime = 1000 -- Force to 1 second if 0
      
      QI(#initTimeout).SetTimeout(lTime * 1000)
      
      if (lEnabled) then
        QI(#status).SetStatus(#admission, "Enabled")
      else
        QI(#status).SetStatus(#admission, "Disabled")
      end if
  end case
end

on _FetchSkinNameCallback (me, theControl, theData)
  if (listP(theControl)) then
    if (theControl.code = "100" or theControl.code = "200") then
      if (theData <> "") then
        -- Parse FetchSkinName answer data
        me.LoadXMLFromString(theData)
        me.Parse(#_FetchSkinNameXMLCallback)
        
        -- Keep going with initialization
        me.FetchUrlList()
        return()
      end if
    end if
  end if
  
  me._InitializationFailed()
end

on _FetchSkinNameXMLCallback (me, theItemName, theItemValue)
  case (theItemName) of
    #skin_name:
      lSkinName = me.GetPathItemValue(theItemValue, "child.value")
      QI(#skin).SetSkinFolder(string(lSkinName))
  end case
end

on _FetchUrlListCallback (me, theControl, theData)
  if (listP(theControl)) then
    if (theControl.code = "100" or theControl.code = "200") then
      if (theData <> "") then
        -- Store the URL list
        QI(#internet).SetInternetXML(theData)
        
        -- Keep going with initialization
        me.FetchTvChannelList()
        return()
      end if
    end if
  end if
  
  me._InitializationFailed()
end

on _FetchTvChannelListCallback (me, theControl, theData)
  if (listP(theControl)) then
    if (theControl.code = "100" or theControl.code = "200") then
      if (theData <> "") then
        -- Store the TV channel list
        --QI(#video).SetVideoXML(theData)
        QI(#opsCom).FakeFetchTvChannelList()
        -- Keep going with initialization
        me.FetchBannerList()
        return()
      end if
    end if
  end if
  
  me._InitializationFailed()
end

on _FetchBannerListCallback (me, theControl, theData)
  if (listP(theControl)) then
    if (theControl.code = "100" or theControl.code = "200") then
      if (theData <> "") then
        -- Store the banner list
        QI(#banner).SetBannerXML(theData)
        
        -- Keep going with initialization
        me.FetchAttractLoopName()
        return()
      end if
    end if
  end if
  
  me._InitializationFailed()
end

on _FetchAttractLoopNameCallback (me, theControl, theData)
  if (listP(theControl)) then
    if (theControl.code = "100" or theControl.code = "200") then
      if (theData <> "") then
        -- Parse FetchSkinName answer data
        me.LoadXMLFromString(theData)
        me.Parse(#_FetchAttractLoopNameXMLCallback)
        
        -- Keep going with initialization
        me.FetchWebProxyList()
        return()
      end if
    end if
  end if
  
  me._InitializationFailed()
end

on _FetchAttractLoopNameXMLCallback (me, theItemName, theItemValue)
  case (theItemName) of
    #attract_loop_name:
      lAttractLoopName = me.GetPathItemValue(theItemValue, "child.value")
      QI(#attractLoop).SetAttractLoopName(string(lAttractLoopName))
  end case
end

on _FetchWebProxyListCallback (me, theControl, theData)
  if (listP(theControl)) then
    if (theControl.code = "100" or theControl.code = "200") then
      if (theData <> "") then
        -- Store the WebProxy list
        QI(#webProxy).SetWebProxyXML(theData)
        
        -- Initialization is done
        me._InitializationCompleted()
        return()
      end if
    end if
  end if
  
  me._InitializationFailed()
end
