-------------------------------------------------------------------------------
-- Name: class NFLGSIS
-- Author: Jonathan Primeau
--
-- Description:
--     This class takes care of the GSIS Flash. It loads its configuration from
--     gsis.xml
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- P R O P E R T I E S
-------------------------------------------------------------------------------

property ancestor              -- Parent
property pOverrideDateCheck
property pSimulDate
property pStubURL
property pStubBrowser
property pScheduleURL
property pScheduleFilename
property pSeasonType
property pHomeTeamCode
property pGameWeekList
property pTeamList

property pKeyList
property pScheduleInfo


-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  -- We need XML Parser
  me.ancestor = script("class XMLParser").new()
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  me.ancestor = void
end

-------------------------------------------------------------------------------
-- void Load (me)
--
-- This method loads the GSIS configuration. You need to call this method to 
-- load the configuration. It parses the GSIS XML and loads it. All 
-- attributes are then avaiable via Get methods.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Load (me)
  GenLog(#debug, "NFLGSIS::Load: Loading...")
  me._ResetXMLProp()
  lGSISPath = "Configuration\nflgsis.xml"
  me.LoadXMLFromFile(lGSISPath)
  me.Parse(#_XMLCallback)
  return()
end

on Refresh (me)
  GenLog(#debug, "NFLGSIS::Refresh: Refreshing...")
  me._ResetScheduleInfo()
  me._RetrieveSchedule()
end

on GetURL (me)
  lYear = me.pScheduleInfo.seasonYear
  lType = me.pScheduleInfo.seasonType
  lWeek = me.pScheduleInfo.weekNumber
  lKey = me.pScheduleInfo.gameKey
  
  lURL = ""
  if (lKey <> "") then
    lURL = me.pStubURL & "Season=" & lYear & "&SeasonType=" & lType & "&Week=" & lWeek & "&GameKey=" & lKey
  end if
  GenLog(#debug, "NFLGSIS::GetURL: Returning:" && lURL)
  return(lURL)
end

-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  me._ResetXMLProp()
  
  -- Build the key list
  me.pKeyList = []
  me.pKeyList.add(#gameKey)
  me.pKeyList.add(#gameDate)
  me.pKeyList.add(#localGameTime)
  me.pKeyList.add(#easternGameTime)
  me.pKeyList.add(#homeClubCode)
  me.pKeyList.add(#homeClubName)
  me.pKeyList.add(#homeClubCity)
  me.pKeyList.add(#homeClubNickName)
  me.pKeyList.add(#homeClubPrimaryColor)
  me.pKeyList.add(#homeClubSecondaryColor)
  me.pKeyList.add(#homeClubColor)
  me.pKeyList.add(#visitingClubCode)
  me.pKeyList.add(#visitingClubName)
  me.pKeyList.add(#visitingClubCity)
  me.pKeyList.add(#visitingClubNickname)
  me.pKeyList.add(#visitingClubPrimaryColor)
  me.pKeyList.add(#visitingClubSecondaryColor)
  me.pKeyList.add(#visitingClubColor)
  
  me._ResetScheduleInfo()
end

on _ResetXMLProp (me)
  me.pOverrideDateCheck = 0
  me.pSimulDate = ""
  me.pStubURL = ""
  me.pStubBrowser = ""
  me.pScheduleURL = ""
  me.pScheduleFilename = ""
  me.pSeasonType = ""
  me.pHomeTeamCode = ""
  me.pGameWeekList = []
  me.pTeamList = []
end

on _ResetScheduleInfo (me)
  me.pScheduleInfo = [:]
  me.pScheduleInfo.addProp(#seasonYear, "")
  me.pScheduleInfo.addProp(#seasontype, "")
  me.pScheduleInfo.addProp(#weekNumber, "")
  me.pScheduleInfo.addProp(#gameKey, "")
end

-------------------------------------------------------------------------------
-- void _XMLCallback (me, theItemName, theItemValue)
--
-- Callback used by the XML parser to return data. This method is the brain of
-- the configuration XML because it knows what it has to look for. 
-- It is specified to the XML Parse method and is called everytime it 
-- encounters an XML tag. If you need to retrieve information from a new tag, 
-- you just have to add the symbol corresponding and use the appropriate path. 
-- You can easily retrieve any data you want using GetItemValue from the XML 
-- parser. Then you have to add the retieved information to the appropriate 
-- list.
--
-- Parameters:
--     me (object): this pointer.
--     theItemName (symbol): The current item name being parsed.
--     theItemValue (list): Contains the information for the current item.
--
-------------------------------------------------------------------------------
on _XMLCallback (me, theItemName, theItemValue)
  case (theItemName) of
    #override_date_check:
      me.pOverrideDateCheck = integer(me.GetPathItemValue(theItemValue, "child.value"))
      
    #simul_date:
      me.pSimulDate = string(me.GetPathItemValue(theItemValue, "child.value"))
      
    #stub_url:
      me.pStubURL = string(me.GetPathItemValue(theItemValue, "child.value"))
      
    #stub_browser:
      me.pStubBrowser = string(me.GetPathItemValue(theItemValue, "child.value"))
      
    #schedule_url:
      me.pScheduleURL = string(me.GetPathItemValue(theItemValue, "child.value"))
      
    #schedule_filename:
      me.pScheduleFilename = string(me.GetPathItemValue(theItemValue, "child.value"))
      
    #season_type:
      me.pSeasonType = string(me.GetPathItemValue(theItemValue, "child.value"))
      
    #home_team_code:
      me.pHomeTeamCode = string(me.GetPathItemValue(theItemValue, "child.value"))
      
    #week:
      lWeekName = string(me.GetAttribute(theItemValue, "name"))
      lWeekType = string(me.GetAttribute(theItemValue, "type"))
      lWeekStart = string(me.GetItemValue(theItemValue, "week_start"))
      lWeekNumber = string(me.GetItemValue(theItemValue, "week_number"))
      
      lCurrentWeek = [:]
      lCurrentWeek.addProp(#name, lWeekName)
      lCurrentWeek.addProp(#type, lWeekType)
      lCurrentWeek.addProp(#start, lWeekStart)
      lCurrentWeek.addProp(#weekNumber, lWeekNumber)
      me.pGameWeekList.add(lCurrentWeek)
      
    #team:
      lTeamShortName = string(me.GetItemValue(theItemValue, "short_name"))
      lTeamCompleteName = string(me.GetItemValue(theItemValue, "complete_name"))
      lTeamName = string(me.GetItemValue(theItemValue, "team_name"))
      lCity = string(me.GetItemValue(theItemValue, "city"))
      lPrimaryColor = string(me.GetItemValue(theItemValue, "primary_color"))
      lSecondaryColor = string(me.GetItemValue(theItemValue, "secondary_color"))
      lColor = string(me.GetItemValue(theItemValue, "color"))
      
      lCurrentTeam = [:]
      lCurrentTeam.addProp(#shortName, lTeamShortName)
      lCurrentTeam.addProp(#completeName, lTeamCompleteName)
      lCurrentTeam.addProp(#teamName, lTeamName)
      lCurrentTeam.addProp(#city, lCity)
      lCurrentTeam.addProp(#primaryColor, lPrimaryColor)
      lCurrentTeam.addProp(#secondaryColor, lSecondaryColor)
      lCurrentTeam.addProp(#color, lColor)
      me.pTeamList.add(lCurrentTeam)
      
  end case
end

on _GetCurrentDate (me)
  -- Are we simulating?
  if (me.pOverrideDateCheck = 0) then
    return(the systemDate)
  else
    return(me._ConvertDate(me.pSimulDate))
  end if
end

on _ConvertDate (me, theDate)
  lYear = integer(theDate.item[1])
  lMonth = integer(theDate.item[2])
  lDay = integer(theDate.item[3])
  lDate = date(lYear, lMonth, lDay)
  return(lDate)
end

on _RetrieveSchedule (me)
  GenLog(#debug, "NFLGSIS::_RetrieveSchedule: Getting NFLGSIS schedule...")
  
  lCurrentDate = me._GetCurrentDate()
  lSeason = lCurrentDate.year
  
  me._GetScheduleInfo()
  lYear = me.pScheduleInfo.seasonYear
  lType = me.pScheduleInfo.seasonType
  lWeek = me.pScheduleInfo.weekNumber
  
  lScheduleStr = me.pScheduleURL & "/" & lYear & "/" & lType & "/" & lWeek & "/" & me.pScheduleFilename
  QI(#netText).Get(lScheduleStr, #_PocessSchedule, me)
end

on _GetScheduleInfo (me)
  
  lOldDelimiter = _player.itemDelimiter
  _player.itemDelimiter = ","
  
  lScheduleInfo = [:]
  
  lCurrentDate = me._GetCurrentDate()
  
  -- Get the first week date
  lFirstWeekDate = me._ConvertDate(me.pGameWeekList[1].start)
  
  -- Check if we are entering before the start of the season
  -- Use the first week of the season
  if (lCurrentDate < lFirstWeekDate) then
    lSeasonYear = lFirstWeekDate.year
    lSeasonType = me.pGameWeekList[1].type
    lWeekNumber = me.pGameWeekList[1].weekNumber
    
  else
    
    lWeekCount = me.pGameWeekList.count
    repeat with i = 1 to lWeekCount
      
      -- Get the current week from the list
      lPrevWeekDate = me._ConvertDate(me.pGameWeekList[i].start)
      
      if (lCurrentDate >= lPrevWeekDate) then
        if (i + 1 <= lWeekCount) then
          -- Get the next week from the list
          lNextWeekDate = me._ConvertDate(me.pGameWeekList[i+1].start)
          
          if (lCurrentDate < lNextWeekDate) then
            lCurrWeek = i
            exit repeat
          end if
        else
          lCurrWeek = lWeekCount
        end if
      end if
    end repeat
    
    lSeasonYear = lFirstWeekDate.year
    lSeasonType = me.pGameWeekList[lCurrWeek].type
    lWeekNumber = me.pGameWeekList[lCurrWeek].weekNumber
  end if
  
  me.pScheduleInfo.seasonYear = lSeasonYear
  me.pScheduleInfo.seasonType = lSeasonType
  me.pScheduleInfo.weekNumber = lWeekNumber
  
  _player.itemDelimiter = lOldDelimiter
end

on _PocessSchedule (me, theStatus, theNetText)
  GenLog(#debug, "NFLGSIS::_PocessSchedule: Status:" && string(theStatus) && "NetText:" && theNetText)
  
  if (theStatus = "Complete") then
    lGameKeyList = me._GenerateGameKeyList(theNetText)
    lGameKeyInfo = me._GetGameInfo(me.pHomeTeamCode, lGameKeyList)
    
    if (listP(lGameKeyInfo)) then
      GenLog(#debug, "NFLGSIS::_PocessSchedule: Setting game info:" && string(lGameKeyInfo))
      
      me.pScheduleInfo.gameKey = lGameKeyInfo.gameKey
    else
      GenLog(#debug, "NFLGSIS::_PocessSchedule: Setting game info with no home game")
      
      lGameKey = string(lGameKeyList.getPropAt(1))
      lGameKeyInfo = lGameKeyList[1]
      me.pScheduleInfo.gameKey = lGameKeyInfo.gameKey
    end if
  else
    me._ResetScheduleInfo()
  end if
end

--Game Key, game date, local game time, Eastern Time game time, home club code, home club name, 
--home club city, home club nickname, home club primary color value, home club secondary color
--value, home club color, visiting club code, visiting club name, visiting
--club city, visiting club nickname, visiting club primary color value,
--visiting club secondary color value, visiting club color.

on _GenerateGameKeyList (me, theData)
  
  lOldDelimiter = _player.itemDelimiter
  _player.itemDelimiter = ""
  
  lGameCount = theData.lines.count
  lGameKeyList = [:]
  lKeyListCount = me.pKeyList.count
  
  repeat with i = 1 to lGameCount
    lCurrGame = theData.line[i]
    
    if (lCurrGame.items.count = lKeyListCount) then
      lTempList = [:]
      repeat with j = 1 to lKeyListCount
        lTempList.addProp(me.pKeyList[j], lCurrGame.item[j])
      end repeat
      lGameKeyList.addProp(integer(lCurrGame.item[1]), lTempList)
    end if
  end repeat
  
  _player.itemDelimiter = lOldDelimiter
  
  return(lGameKeyList)
end

on _GetGameInfo (me, theClubCode, theGameKeyList)
  lCount = theGameKeyList.count
  repeat with i = 1 to lCount
    if (theGameKeyList[i].homeClubCode = theClubCode or theGameKeyList[i].visitingClubCode = theClubCode) then
      return(theGameKeyList[i])
    end if
  end repeat
  
  return(-1)
end