-------------------------------------------------------------------------------
-- Name: class Navigation
-- Author: Jonathan Primeau
--
-- Description:
--     This class is the heart of navigation. Everytime the user navigates,
--     the Navigate method is called. It also manages history.
--
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- P R O P E R T I E S
-------------------------------------------------------------------------------
property ancestor              -- Parent
property pHistory
property pAction
property pTargetMarker
property pSupportedSection
property pSupportedFunction
property pServerNavigateTarget
-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = script("class XMLParser").new()
  
  -- Initialize
  me._Init()
  
  return(me)
end
-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  me.ancestor = void
end
-------------------------------------------------------------------------------
-- void Navigate (me, theNavProps, theAddToHistory)
--
-- This is the main function to navigate. It logs the navigation in the 
-- log for future reference (debugging). It also adds the navigation into the
-- history list (if desired). Basically, this is the only place where we should
-- use go to frame except when looping frame
--
-- Parameters:
--     me (object): this pointer.
--     theNavProps (list): Navigation properties (section, parameters)
--     theAddToHistory (bool): Add navigation to history
--
-------------------------------------------------------------------------------
on Navigate (me, theTarget, theAddHistory, theWebClick)
  
  if (theWebClick = void) then theWebClick = false
  
  me.pTargetMarker = ""
  
  -- Execute leaving section script
  LeavingSection(QI(#status).GetStatus().area)
  
  GenLog(#debugnav, "Navigation::Navigate:" && theTarget)
  
  if (theAddHistory = void) then theAddHistory = false
  me.pAction = [:]
  me.pAction.addProp(#target, theTarget)
  me.pAction.addProp(#history, theAddHistory)
  
  lStatus = me._ParseTarget(theTarget, theWebClick)
  if (lStatus <= 0) then
    -- < 0: An error occured
    -- = 0: Ignore navigation
    return(lStatus)
  end if
  
  -- Add to history
  if (me.pAction.history = true) then
    me.AddToHistory(me.pAction.target)
    me.pAction.history = false
  end if
  
  if (theWebClick = false or theTarget starts "ogn://" = true) then
    -- Do the navigation only if it's not a web click
    --QI(#navTimeout).TriggerNavigation()
  end if
  
  ActLog(#navigation, "User", theTarget)
  
  return(1)
end
on ServerNavigate (me, theData)
  lTarget = ""
  lAddToHistory = false
  
  me.LoadXMLFromString(theData)
  lStatus = me.Parse(#_ServerNavigateXMLCallback)
  if (lStatus <> 0) then
    GenLog(#error, "Navigation::ServerNavigate: Invalid target sent by the server:" && me.pServerNavigateTarget)
    SvrLog("Navigate: Invalid target sent by the server:" && me.pServerNavigateTarget)
    me.pServerNavigateTarget = ""
    return(-1)
  end if
  
  lTarget = me.pServerNavigateTarget
  
  ActLog(#navigation, "Administrator", lTarget)
  
  me.Navigate(lTarget, lAddToHistory)
  
  return(0)
end
-------------------------------------------------------------------------------
-- void GoBack (me)
--
-- Goes back in main navigation.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on GoBack (me, theForceBannerCycling)
  lValue = me.pHistory.GoBack()
  
  if (lValue <> void) then
    GenLog(#debugnav, "Navigation::GoBack:" && lValue)
    me.Navigate(lValue, false)
  end if
  
  if (theForceBannerCycling = true) then
    QI(#banner).Cycle()
  end if
end
-------------------------------------------------------------------------------
-- void GoForward (me)
--
-- Goes forward in main navigation
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on GoForward (me)
  lValue = me.pHistory.GoForward()
  
  if (lValue <> void) then
    GenLog(#debugnav, "Navigation::GoForward:" && lValue)
    me.Navigate(lValue, false)
  end if
end
on AddToHistory (me, theTarget)
  me.pHistory.AddToHistory(theTarget)
end
on ClearHistory (me)
  me.pHistory.ClearHistory()
end
on GetAction (me)
  return(me.pAction)
end
on GetParams (me)
  lParams = [:]
  if (me.pAction.getaProp(#params) <> void) then
    lParams = me.pAction.params
  end if
  return(lParams)
end
on DoNavigate (me)
  if (me.pTargetMarker <> "") then
    go to frame me.pTargetMarker
  end if
end
-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  me.pHistory = script("class History").new()
  me.pAction = [:]
  
  me.pSupportedSection = GetMarkerList()
  
  me.pSupportedFunction = []
  me.pSupportedFunction.add("fb") -- food and beverage
  me.pSupportedFunction.add("back") -- back
  me.pSupportedFunction.add("begintryout") -- begin tryout
  me.pSupportedFunction.add("dologout") -- do logout
  me.pSupportedFunction.add("displayMaintenance") -- Displays the maintenance screen
  me.pSupportedFunction.add("exit") -- Exit the kiosk
  me.pSupportedFunction.add("nflgsis") -- NFL GSIS
  
  me.pServerNavigateTarget = ""
  me.pTargetMarker = ""
end
on _ParseTarget (me, theTarget, theWebClick)
  GenLog(#debugnav, "Navigation::ParseTarget: Parsing:" && theTarget & "...")
  
  lStatus = 0
  
  if (theTarget starts "ogn://") then
    
    GenLog(#debugnav, "Navigation::ParseTarget: Target is Navigate!")
    me.pAction.addProp(#type, #navigate)
    
    lOldDelimiter = _player.itemDelimiter
    _player.itemDelimiter = "/"
    
    lCount = theTarget.item.count
    
    if (lCount <> 4) then
      GenLog(#error, "Navigation::ParseTarget: Invalid target format!")
      SvrLog("Navigate: Invalid target format")
      me.pAction = [:]
      _player.itemDelimiter = lOldDelimiter
      return(-101)
    end if
    
    lAction = theTarget.item[3]
    lInfo = theTarget.item[4]
    
    _player.itemDelimiter = "?"
    
    lCount = lInfo.item.count
    
    if (lCount < 1) then
      GenLog(#error, "Navigation::ParseTarget: Invalid target information!")
      SvrLog("Navigate: Invalid target information")
      me.pAction = [:]
      _player.itemDelimiter = lOldDelimiter
      return(-102)
    end if
    
    lSection = lInfo.item[1]
    
    lIsSection = false
    lIsFunction = false
    if (me.pSupportedSection.getPos(lSection) <> 0) then
      me.pAction.addProp(#section, lSection)
      lIsSection = true
    end if
    
    if (me.pSupportedFunction.getPos(lSection) <> 0) then
      me.pAction.addProp(#function, lSection)
      lIsFunction = true
    end if
    
    if (lIsSection = false and lIsFunction = false) then
      GenLog(#error, "Navigation::ParseTarget: Invalid section or function!")
      SvrLog("Navigate: Invalid section or function")
      me.pAction = [:]
      _player.itemDelimiter = lOldDelimiter
      return(-103)
    end if
    
    lCurrentParams = [:]
    if (lCount > 1) then
      
      lParams = lInfo.item[2]
      
      _player.itemDelimiter = "&"
      
      lCount = lParams.item.count
      
      lParamList = []
      repeat with i = 1 to lCount
        lParamList.add(lParams.item[i])
      end repeat
      
      _player.itemDelimiter = "="
      
      repeat with i = 1 to lCount
        lParamItemCount = lParamList[i].item.count
        if (lParamItemCount <> 2) then
          GenLog(#error, "Navigation::ParseTarget: Invalid parameter!")
          SvrLog("Navigate: Invalid parameter")
          me.pAction = [:]
          _player.itemDelimiter = lOldDelimiter
          return(-104)
        end if
        lParamName = symbol(lParamList[i].item[1])
        lParamValue = string(lParamList[i].item[2])
        lCurrentParams.addProp(lParamName, lParamValue)
      end repeat
      
    end if
    me.pAction.addProp(#params, lCurrentParams)
    
    _player.itemDelimiter = lOldDelimiter
    
    lStatus = me._DoNavigation(theWebClick)
    
  else
    GenLog(#debugnav, "Navigation::ParseTarget: Target is URL!")
    
    lTarget = string(theTarget)
    
    -- Food and beverage: add the kiosk location
    if (lTarget = QI(#config).GetFoodBeverageURL()) then
      lLocation = QI(#status).GetLocation()
      lTarget = lTarget & "?kn=" & lLocation
    end if
    
    
    if (QI(#status).GetStatus().area <> "web_admin") then
      lBrowser = QI(#webProxy).Try(lTarget)
    else
      lBrowser = "WebAdmin"
    end if
    
    me.pAction.addProp(#type, #internet)
    me.pAction.addProp(#url, lTarget)
    me.pAction.addProp(#browser, lBrowser)
    
    lStatus = me._DoNavigation(theWebClick)
  end if
  
  return(lStatus)
end
on _DoNavigation (me, theWebClick)
  -- = 1: Navigate
  -- = 0: Don't navigate
  -- < 0: Error
  
  lStatus = true
  
  if (me.pAction <> [:]) then
    
    if (me.pAction.type = #internet) then
      
      -- Bail if we don't have the browser
      if (me.pAction.getaProp(#browser) = void) then
        return(-201)
      end if
      
      GenLog(#debugnav, "Navigation::_DoNavigation:" && me.pAction.browser && me.pAction.url)
      
      ActLog(#access, "URL", me.pAction.browser & "," && me.pAction.url)
      
      -- Check WebProxy policy
      if (me.pAction.browser = "WebXtra") then
        -- Use web xtra
        lTargetMarker = "internet_WebXtra"
        
      else if (me.pAction.browser = "WebXtraNoNavigation") then
        -- Use web xtra with no toolbar
        lTargetMarker = "internet_WebXtra"
        me.pAction.addProp(#toolbar, false)
        
      else if (me.pAction.browser = "WebViewerXtra") then
        -- Use web viewer xtra
        lTargetMarker = "internet_WebViewerXtra"
        
      else if (me.pAction.browser = "WebViewerXtraNoNavigation") then
        -- Use web viewer xtra with no toolbar
        lTargetMarker = "internet_WebViewerXtra"
        me.pAction.addProp(#toolbar, false)
        
      else if (me.pAction.browser = "MozillaXtra") then
        -- Use mozilla xtra with toolbar
        lTargetMarker = "internet_MozillaXtra"
        me.pAction.addProp(#toolbar, true)
        
      else if (me.pAction.browser = "MozillaXtraSimple") then
        -- Use mozilla xtra using a special agent and toolbar
        lTargetMarker = "internet_MozillaXtra"
        lUserAgent = QI(#config).GetLowEndUserAgent()
        me.pAction.addProp(#userAgent, lUserAgent)
        me.pAction.addProp(#toolbar, true)
        
      else if (me.pAction.browser = "MozillaXtraNoNavigation") then
        -- Use mozilla xtra with no toolbar
        lTargetMarker = "internet_MozillaXtra"
        me.pAction.addProp(#toolbar, false)
        
      else if (me.pAction.browser = "Ignore") then
        -- Bail
        return(false)
        
      else if (me.pAction.browser = "Deny") then
        -- Show the denied message and bail out
        QI(#webDenied).Show()
        return(false)
        
      else if (me.pAction.browser = "WebAdmin") then
        -- Web administration
        lTargetMarker = "web_admin"
        
      end if
      
      -- Check the switch browser policy
      lCurrBrowser = QI(#status).GetCurrentBrowser()
      if (me.pAction.browser <> lCurrBrowser) then
        lSwitchPolicy = QI(#config).GetSwitchBrowserPolicy()
        if (lSwitchPolicy = "deny") then
          QI(#webDenied).Show()
          return(false)
        else if (lSwitchPolicy = "continue") then
          return(true)
        else if (lSwitchPolicy = "ignore") then
          return(false)
        end if
      end if
      
      me.pTargetMarker = lTargetMarker
      if (theWebClick = false) then
        go to frame me.pTargetMarker
      end if
      
    else if (me.pAction.type = #navigate) then
      
      if (me.pAction.getaProp(#section) <> void) then
        lTargetMarker = me.pAction.section
        
        -- Check with the capabilities if we are allowed to go there
        if (QI(#cap).Try(me.pAction.section) = false) then
          ActLog(#access, "MainNavigation", "Disabled," && lTargetMarker)
          lTargetMarker = "ogn://navigate/disabled"
          me.Navigate(lTargetMarker, true)
          return(false)
        end if
        
        me.pTargetMarker = lTargetMarker
        go to frame me.pTargetMarker
      else if (me.pAction.getaProp(#function) <> void) then
        lFunction = me.pAction.function
        me._DoFunction(lFunction)
      end if
      
    end if
  end if
  
  return(lStatus)
end
on _DoFunction (me, theFunction)
  case (theFunction) of
    "fb":
      lFoodBeverageURL = QI(#config).GetFoodBeverageURL()
      me.Navigate(lFoodBeverageURL, true)
      
    "back":
      me.pAction.history = false -- Force
      me.GoBack()
      
    "begintryout":
      QI(#login).BeginTryout()
      
    "dologout":
      -- JP NO ENTERBET
      QI(#login).LogoutNoEnterbet()
      
    "displayMaintenance":
      me.Navigate("ogn://navigate/maintenance", false)
      
    "exit":
      halt()
      
    "nflgsis":
      lNFLgsisURL = QI(#nflgsis).GetURL()
      me.Navigate(lNFLgsisURL, true)
      
  end case
end
on _ServerNavigateXMLCallback (me, theItemName, theItemValue)
  case (theItemName) of
      
    #target:
      me.pServerNavigateTarget = string(me.GetPathItemValue(theItemValue, "child.value"))
      
  end case
end
on IsCurrentlyInWebBrowser (me)
  lCurrentSection = QI(#status).GetStatus().area
  
  if (lCurrentSection starts "internet_") then
    return(true)
  else
    return(false)
  end if
end