-------------------------------------------------------------------------------
-- Name: class NetText
-- Author: Jonathan Primeau
--
-- Description:
--     This class offers an API for getting text from an URL
--
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- P R O P E R T I E S
-------------------------------------------------------------------------------

property ancestor              -- Parent
property pNetID                -- Current NetText ID
property pURL                  -- Current URL for NetText (source)
property pCallback             -- Current callback information
property pStatus               -- The NetText status


-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = script("class NetTextTimeout").new()
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  me.ancestor = void
end

-------------------------------------------------------------------------------
-- void Get (me)
--
-- This methods is called when the net text is successful. It stops the check
-- net text status timeout, adds the net text to the cache and calls the 
-- callback specified when the net text starts.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Get (me, theURL, theCallback, theObject)
  
  -- TODO: Validate parameters
  
  -- Keep track of parameters for later use
  me.pStatus = ""
  me.pURL = theURL
  me.pCallback = [#handler: theCallback, #object: theObject]
  
  -- Starting get net text...
  GenLog(#debug, "NetText::Get: Getting text from" && theURL)
  me.pNetID = getNetText(theURL)
  
  -- Start the check net text status timeout
  me.Start()
end

-------------------------------------------------------------------------------
-- void Done (me)
--
-- This methods is called when the net text is successful. It stops the check
-- net text status timeout, adds the net text to the cache and calls the 
-- callback specified when the net text starts.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Done (me)
  -- Stop the check net text status timeout
  me.Stop()
  
  GenLog(#debug, "NetText::Sone: Done getting text!")
  lNetText = NetTextResult(me.pNetID)
  
  -- Done! Give the hand to our caller
  me._CallCallback(lNetText)
end

-------------------------------------------------------------------------------
-- void NetTextFailed (me)
--
-- This methods is called when the net text failed. It simply calls the 
-- callback specified when the net text starts.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Failed (me)
  -- Stop the check net text status timeout
  me.Stop()
  
  GenLog(#error, "NetText::Failed: Net Text failed NetError:" && me.pStatus)
  
  -- Failed! Give the hand to our caller
  me._CallCallback("")
end

-------------------------------------------------------------------------------
-- void CheckDone (me)
--
-- Checks if the current net text is done. This is called by 
-- CheckStatus periodically.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     True if the net text is done.
--
-------------------------------------------------------------------------------
on CheckDone (me)
  if (NetDone(me.pNetID)) then
    me.Done()
    return(true)
  end if
  return(false)
end

-------------------------------------------------------------------------------
-- void CheckStatus (me)
--
-- Checks if the current net text status is OK.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     True if the net text is done.
--
-------------------------------------------------------------------------------
on CheckStatus (me)
  me.pStatus = GetStreamStatus(me.pNetID).state

  case me.pStatus of
    "Connecting":
      GenLog(#debug, "NetText::CheckStatus: Connecting")
    "Started":
      GenLog(#debug, "NetText::CheckStatus: Started")
    "InProgress":
      GenLog(#debug, "NetText::CheckStatus: InProgress")
    "Error":
      GenLog(#debug, "NetText::CheckStatus: Error")
      lStatus = NetError(me.pNetID)
      me.Failed()
    "NoInformation":
      GenLog(#debug, "NetText::CheckStatus: NoInformation")
      me.Failed()
    "Complete":
      GenLog(#debug, "NetText::CheckStatus: Complete")
  end case
  return(me.pStatus)
end

-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  me.pNetID = 0
  me.pStatus = ""
  me.pURL = ""
  me.pCallback = [#handler: void, #object: void]
end

-------------------------------------------------------------------------------
-- void _CallCallback (me)
--
-- Calls the callback specified when the net text starts. The same callback is
-- called whether the retrieving succeeded or not. However, the callback 
-- handler should expect some parameters. The first one is the net text status
-- (0 for success and different than 0 for error).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _CallCallback (me, theNetText)
  -- Call the specified callback
  lHandler = me.pCallback.handler
  lObject = me.pCallback.object
  lStatus = me.pStatus
  
  if (symbolP(lHandler) = 0) then
    return()
  end if
  
  if (objectP(lObject) = 0) then
    return()
  end if
  
  call(lHandler, lObject, lStatus, theNetText)
end
