-------------------------------------------------------------------------------
-- Name: class OPSConnectionTimeout
-- Author: Jonathan Primeau
--
-- Description:
--     Takes care of the connection with the Operation Server.
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- P R O P E R T I E S
------------------------------------------------------------------------------- 

property ancestor              -- Parent
property pNumberRetry
property pTimeout


-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  --me.ancestor = script("class Timeout").new()
  me.ancestor = script("class XMLParser").new()
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  
  me.ancestor = void
  
  me.pTimeout = void
end

on Start (me)
  GenLog(#debug, "OPSConnectionTimeout::Start: Starting timer...")
  
  me.pTimeout.ResetHit()
  me.pTimeout.SetName("to,OPSConnection")
  lOPSInfo = QI(#config).GetOPSInfo()
  lRetry = lOPSInfo.wait_retry * 1000
  me.pTimeout.SetHitScale(lRetry)
  
  me.pTimeout.SetTimeout(0) -- Never timing out
  
  me.pTimeout.SetHitCallback(#_HitCallback, me)
  me.pTimeout.SetTimeoutCallback(#_TimeoutCallback, me)
  
  if (me.pTimeout.Start() = true) then
    GenLog(#debug, "OPSConnectionTimeout::Start: Timer started!")
  else 
    GenLog(#error, "OPSConnectionTimeout::Start: Timer NOT started!")
  end if
end

on Stop (me)
  GenLog(#debug, "OPSConnectionTimeout::Stop: Stopping timer...")
  me.pTimeout.Stop()
end


-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  me.pTimeout = script("class Timeout").new()
  me.pNumberRetry = 0
end

on _HitCallback (me)
  -- If we are not connected to the OPS
  if (QI(#opsCom).IsConnected() = false) then
    
    lOPSInfo = QI(#config).GetOPSInfo()
    lLimitNumberRetry = lOPSInfo.number_retry
    
    if (lLimitNumberRetry = 0) then
      
      -- If the limit is zero, don't retry
      QI(#fatalError).Show()
      QI(#terminalCom).EndPatron()
      QI(#init).ConnectionTimeout()
      
    else
      
      -- One more try
      me.pNumberRetry = me.pNumberRetry + 1
      
      -- OK, here is our last chance, let the user know
      if (me.pNumberRetry = lLimitNumberRetry - 1) then
        QI(#fatalError).Show()
      end if
      
      -- If we exceeded the maximum number of retry, timeout
      if (me.pNumberRetry >= lLimitNumberRetry) then
        QI(#terminalCom).EndPatron()
        QI(#init).ConnectionTimeout()
      else
        if (QI(#init).IsInitialized() = true and QI(#init).IsLoaded() = true) then
          -- Try to connect again with the next OPS slot
          QI(#opsCom).Connect(#next, #_TryConnectCallback, me)
        else
          -- Just restart
          QI(#init).ConnectionTimeout()
        end if
      end if      
    end if
  else
    -- Everything is fine
    me.pNumberRetry = 0
  end if
end

on _TimeoutCallback (me)
  
end

on _TryConnectCallback (me)
  QI(#opsCom).ResumeKiosk(#_ResumeKioskCallback, me)
end

on _ResumeKioskCallback (me, theControl, theData)
  if (listP(theControl)) then
    if (theControl.code = "100" or theControl.code = "200") then
      if (theData <> "") then
        -- Parse InitializeKiosk answer data
        me.LoadXMLFromString(theData)
        lStatus = me.Parse(#_ResumeKioskXMLCallback)
        
        if (lStatus <> 0) then
          GenLog(#error, "OPSConnectionTimeout::_ResumeKioskCallback: Bad XML from OPS")
        end if
        
      end if
    end if
  end if
end

on _ResumeKioskXMLCallback (me, theItemName, theItemValue)
  case (theItemName) of
    #info:
      lKioskID = me.GetItemValue(theItemValue, "kiosk_id")
      lKioskName = me.GetItemValue(theItemValue, "name")
      lGroup = me.GetItemValue(theItemValue, "group")
      lLocation = me.GetItemValue(theItemValue, "location")
      lEnabled = value(me.GetItemValue(theItemValue, "enabled"))
      lTime = integer(me.GetItemValue(theItemValue, "time"))
      
      if (lKioskID = void) then
        lKioskID = QI(#config).GetKioskID()
      end if
      QI(#status).SetKioskID(lKioskID)
      QI(#status).SetKioskName(lKioskName)
      QI(#status).SetGroup(lGroup)
      QI(#status).SetLocation(lLocation)
  end case
end