-------------------------------------------------------------------------------
-- Name: class Patron
-- Author: Jonathan Primeau
--
-- Description:
--     This class takes care of the patron information (enterbet)
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- P R O P E R T I E S
------------------------------------------------------------------------------- 

property ancestor              -- Parent
property pIsForced
property pLastBeginPatronError
property pLastEndPatronError
property pWaitingSwipeCard
property pCardInfo
property pQuickBetsInfo
property pPatronInfo
property pTerminalStatus

-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = script("class XMLParser").new()
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  
  me.ancestor = void
end

on Status (me, theContent)
  GenLog(#debug, "Patron::Status: Loading XML...")
  
  -- Parse BeginPatron answer data
  me.LoadXMLFromString(theContent)
  lStatus = me.Parse(#_StatusXMLCallback)
  
  GenLog(#debug, "Patron::Status: Done loading XML!")
  
  if (lStatus <> 0) then
    GenLog(#error, "Patron::Status: Invalid Status!")
  else
    if (me.pTerminalStatus = true) then
      GenLog(#debug, "Patron::Status: Terminal status is OK!")
      -- Hide Terminal Error
      QI(#terminalError).Hide()
    else
      GenLog(#debug, "Patron::Status: Terminal status is ERROR!")
      -- Show Terminal Error
      QI(#terminalError).Show()
    end if
  end if
end

on BeginPatron (me, theContent)
  -- We are only concerned when the kiosk is in run mode
  if (QI(#init).IsLoaded() and QI(#init).IsInitialized()) then
    
    GenLog(#debug, "Patron::BeginPatron: Loading XML...")
    
    -- Parse BeginPatron answer data
    me.LoadXMLFromString(theContent)
    lStatus = me.Parse(#_BeginPatronXMLCallback)
    
    GenLog(#debug, "Patron::BeginPatron: Done loading XML!")
    
    -- Check for errors
    if (me.pLastBeginPatronError <> "") then
      GenLog(#error, "Patron::BeginPatron: Error:" && me.pLastBeginPatronError)
      ActLog(#patron, "Login", "Invalid login:" && me.pLastBeginPatronError)
      QI(#login).SetDoBeginSession(false)
      return(-101)
    end if
    
    if (lStatus <> 0) then
      GenLog(#error, "Patron::BeginPatron: Invalid BeginPatron!")
    else
      if (me.pIsForced = false) then
        GenLog(#debug, "Patron::BeginPatron: force=false")
        QI(#login).SetDoBeginSession(true)
      else
        GenLog(#debug, "Patron::BeginPatron: force=true")
        -- If the login is already displayed, hide it
        if (QI(#ebLogin).IsVisible() = true) then
          QI(#ebLogin).Hide()
        end if
      end if
    end if
    
  else
    GenLog(#debug, "Patron::BeginPatron: BeginPatron while initializing, ignoring!")
  end if
  return(0)
end

on EndPatron (me, theContent)
  -- We are only concerned when the kiosk is in run mode
  if (QI(#init).IsLoaded() and QI(#init).IsInitialized()) then
    
    GenLog(#debug, "Patron::EndPatron: Loading XML...")
    
    -- Parse EndPatron answer data
    me.LoadXMLFromString(theContent)
    lStatus = me.Parse(#_EndPatronXMLCallback)
    
    GenLog(#debug, "Patron::EndPatron: Done loading XML!")
    
    if (me.pLastEndPatronError starts "System not available") then
      return()
    end if
    
    if (lStatus <> 0) then
      GenLog(#error, "Patron::EndPatron: Invalid EndPatron!")
    else
      if (me.pIsForced = false) then
        GenLog(#debug, "Patron::EndPatron: force=false")
        QI(#login).SetDoBeginSession(false)
        QI(#login).EndSession()
        ActLog(#patron, "Logout", "Patron logged out (from TCS mode)")
      else
        GenLog(#debug, "Patron::EndPatron: force=true")
      end if
    end if
  else
    GenLog(#debug, "Patron::EndPatron: EndPatron while initializing, ignoring!")
  end if
end

on SetWaitingSwipeCard (me, theState)
  me.pWaitingSwipeCard = theState
end

on CheckSwipeCard (me, theTarget)
  -- Credit card in food and beverage
  lTarget = string(theTarget)
  lSwipeURLList = QI(#config).GetSwipeCardURL()
  lSwipeURLCount = lSwipeURLList.count
  
  lSwipeCard = false
  repeat with i = 1 to lSwipeURLCount
    lCurrURL = lSwipeURLList[i]
    if (lTarget starts lCurrURL) then
      lSwipeCard = true
      exit repeat
    end if
  end repeat
  
  me.SetWaitingSwipeCard(lSwipeCard)
end

on SwipeCard (me, theContent)
  GenLog(#debug, "Patron::SwipeCard: Card swipe occured...")
  if (me.pWaitingSwipeCard = true) then
    me.pCardInfo = ""
    
    -- Parse SwipeCard answer data
    me.LoadXMLFromString(theContent)
    lStatus = me.Parse(#_SwipeCardXMLCallback)
    
    QI(#web).MozillaSetEditField("SwipeData", me.pCardInfo, "")
    QI(#web).OpenPage("javascript: submitForm();")
    
    GenLog(#debug, "Patron::SwipeCard: Card swipe processed!")
  else
    GenLog(#debug, "Patron::SwipeCard: Card swipe ignored!")
  end if
end

on QuickBets (me, theContent)
  -- We are only concerned when the kiosk is in run mode
  -- And also only when a patron is logged in
  if (QI(#init).IsLoaded() and QI(#init).IsInitialized() and me.GetPatronInfo().id > 0) then
    
    GenLog(#debug, "Patron::QuickBets: Loading XML...")
    
    -- Parse BeginPatron answer data
    me.LoadXMLFromString(theContent)
    lStatus = me.Parse(#_QuickBetsXMLCallback)
    
    GenLog(#debug, "Patron::QuickBets: Done loading XML!")
    
    if (lStatus <> 0) then
      GenLog(#error, "Patron::QuickBets: Invalid QuickBets!")
    else
      
      QI(#crm).Load("QuickBets")
      
    end if
  else
    GenLog(#debug, "Patron::_QuickBets: QuickBets while initializing, ignoring!")
  end if
end

on Wager (me, theContent)
  -- We are only concerned when the kiosk is in run mode
  if (QI(#init).IsLoaded() and QI(#init).IsInitialized()) then
    GenLog(#debug, "Patron::Wager: Sending Wager XML to OPS...")
    QI(#opsCom).FillCrmOfferWager(theContent)
  end if
end

on SetPatronInfo (me, theInfo)
  me.pPatronInfo = theInfo
end

on SetPatronLogout (me)
  lPatronInfo = [:]
  lPatronInfo.addProp(#account, 0)
  lPatronInfo.addProp(#id, 0)
  lPatronInfo.addProp(#player, 0)
  lPatronInfo.addProp(#name, "")
  lPatronInfo.addProp(#password, "")
  
  me.SetPatronInfo(lPatronInfo)
end

on SetPatronTryout (me)
  lPatronInfo = [:]
  lPatronInfo.addProp(#account, -1)
  lPatronInfo.addProp(#id, -1)
  lPatronInfo.addProp(#player, -1)
  lPatronInfo.addProp(#name, "")
  lPatronInfo.addProp(#password, "")
  
  me.SetPatronInfo(lPatronInfo)
end

on GetPatronInfo (me)
  return(me.pPatronInfo)
end

on GetQuickBetsInfo (me)
  return(me.pQuickBetsInfo)
end


-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  me.pIsForced = false
  me.pLastBeginPatronError = ""
  me.pLastEndPatronError = ""
  me.pWaitingSwipeCard = false
  
  me.pCardInfo = ""
  me.pQuickBetsInfo = [#race: void, #event: void]
  me.pTerminalStatus = false
  
  me.SetPatronLogout()
end


on _BeginPatronXMLCallback (me, theItemName, theItemValue)
  case (theItemName) of
    #beginpatron:
      lForced = me.GetAttribute(theItemValue, "forced")
      lError = me.GetAttribute(theItemValue, "error")
      
      lAccount = me.GetAttribute(theItemValue, "account")
      lID = me.GetAttribute(theItemValue, "id")
      lPlayer = me.GetAttribute(theItemValue, "player")
      lName = me.GetAttribute(theItemValue, "name")
      lPassword = me.GetAttribute(theItemValue, "password")
      
      me.pLastBeginPatronError = string(lError)
      if (lForced = void) then lForced = 0
      me.pIsForced = integer(lForced)
      
      lPatronInfo = [:]
      lPatronInfo.addProp(#account, lAccount)
      lPatronInfo.addProp(#id, lID)
      lPatronInfo.addProp(#player, lPlayer)
      lPatronInfo.addProp(#name, lName)
      lPatronInfo.addProp(#password, lPassword)
      
      me.SetPatronInfo(lPatronInfo)
  end case
end

on _EndPatronXMLCallback (me, theItemName, theItemValue)
  case (theItemName) of
    #endpatron:
      lForced = me.GetAttribute(theItemValue, "forced")
      lError = me.GetAttribute(theItemValue, "error")
      
      lID = me.GetAttribute(theItemValue, "id")
      
      me.pLastEndPatronError = string(lError)
      if (lForced = void) then lForced = 0
      me.pIsForced = integer(lForced)
  end case
end

on _SwipeCardXMLCallback (me, theItemName, theItemValue)
  case (theItemName) of
    #magcard:
      me.pCardInfo = string(me.GetPathItemValue(theItemValue, "child.value"))
  end case
end

on _QuickBetsXMLCallback (me, theItemName, theItemValue)
  case (theItemName) of
    #quickbets:
      lEvent = string(me.GetAttribute(theItemValue, "event"))
      lRace = string(me.GetAttribute(theItemValue, "race"))
      
      if (lEvent = "") then lEvent = "0"
      if (lRace = "") then lRace = "0"
      
      me.pQuickBetsInfo = [:]
      me.pQuickBetsInfo.addProp(#event, lEvent)
      me.pQuickBetsInfo.addProp(#race, lRace)
  end case
end

on _StatusXMLCallback (me, theItemName, theItemValue)
  case (theItemName) of
    #status:
      lStatus = integer(me.GetAttribute(theItemValue, "communications"))
      
      me.pTerminalStatus = lStatus
  end case
end