-------------------------------------------------------------------------------
-- Name: class OPSCommunicator
-- Author: Jonathan Primeau
--
-- Description:
--     This is the layer between MUSClient and high level calls.
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- P R O P E R T I E S
-------------------------------------------------------------------------------

property pNetId
property pURL
property pJSessionId
property pLoginInProgress

property pPollingPeriod
property pPollTimer
property pLoginTimeout
property pLoginTimer

-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

on new (me)
  comInfo = QI(#config).GetSciGamesInfo()
  pURL = comInfo.url
  
  pPollingPeriod = comInfo.getaProp(#pollPeriod)
  pLoginTimeout = comInfo.getaProp(#loginTimeout)
  pPollTimer = script("class Timeout").new()
  pLoginTimer = script("class Timeout").new()
  
  snxProxyServer(#Default)
  pNetId = VOID
  return(me)
end

on Destroy (me)
  
end

on KeepAlive(me)
  me._PollTimerStart()
end

on GetUrl(me)
  return me.pURL
end

on GetJSessionID(me)
  return me.pJSessionId
end

on NetAbort(me)
  if not voidp(pNetId) then
    snxNetAbort(pNetId)
  end if
end

on AbortLogin(me)
  me.NetAbort()
  me._LoginTimerStop()
end

on Post(infolist)
  pNetId = snxPostNetText(pURL, infoList)
end

on LoginChallenge (me, id, PIN)
  
  if (id = 0 AND PIN = 0) then
    --Fake Response when servlet server down
    QI(#SCGLogin).LoginSuccess()
  else
    GenLog(#info, "SCGCommunicator::LoginChallenge: Validating login on Server with this user ID:" && id)
    infoList = ["action":"login", "username":id, "PIN":PIN]
    Post(infoList)
    me._LoginTimerStart()
  end if  
  
end

on Poll (me)
  GenLog(#info, "SCGCommunicator::Poll: polling Server to keep JSessionID Alive:" && pJSessionId)
  infoList = ["action":"poll", "JSESSIONID":pJSessionId]
  Post(infoList)
end

on LogoutChallenge (me)
  me._PollTimerStop()
  GenLog(#info, "SCGCommunicator::LogoutChallenge: Notify Server of User Logout")
  infoList = ["action":"logout"]
  Post(infoList)
end

on Listen (me)
  if not(voidP(pNetId)) then
    
    if (snxNetDone(pNetId) = TRUE) then
      lresult = STRING(snxNetTextResult(pNetId))
      lerror = snxNetError(pNetId)
      GenLog(#info, "SCGCommunicator::Listen: Result:" && lresult)
      GenLog(#info, "SCGCommunicator::Listen: Error:" && lerror)
      
      if lresult <> "" then
        -- Set to Void to stop this loop
        pNetId = VOID
        --Parse result
        me._MessageHandler(lresult)
      end if
      
    end if
  end if
  
end

-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------

on _MessageHandler (me, theResult)
  -- We received a message from the SCG, parse it.
  lControlData = me._SplitControlData(theResult)
  
  if not(voidP(lControlData[#JSESSIONID])) then 
    pJSessionId = lControlData[#JSESSIONID]
  end if
  lResponse = lControlData[#RESPONSE]
  lMessage = lControlData[#MESSAGE]
  
  if (lResponse = -1) then
    GenLog(#info, "SCGCommunicator::Response: -1::Server Acknowledge Logout")
    pJSessionId = VOID
    me._PollTimerStop()
    
  else if (lResponse = 21) then
    GenLog(#info, "SCGCommunicator::Response: 21::Server Responded to Keep Alive poll")
    
  else if (lResponse = 0) then 
    GenLog(#info, "SCGCommunicator::Response: 0::Successul Login")
    me._LoginTimerStop()
    QI(#SCGlogin).LoginSuccess()
    
  else if (lResponse = 1) then
    GenLog(#info, "SCGCommunicator::Response: 1::Invalid account ID PIN")
    me._LoginTimerStop()
    QI(#SCGlogin).LoginFailed("Invalid account ID PIN")
    
  else if (lResponse = 3) then
    GenLog(#info, "SCGCommunicator::Response: 3::Invalid PIN")
    me._LoginTimerStop()
    QI(#SCGlogin).LoginFailed("Invalid PIN")
    
  else if (lResponse = 4) then
    GenLog(#info, "SCGCommunicator::Response: 4::Account ID not numeric")
    me._LoginTimerStop()
    QI(#SCGlogin).LoginFailed("Account ID not numeric")
    
  else if (lResponse = 31) then
    GenLog(#info, "SCGCommunicator::Response: 31::Tote Server Unavailable")
    me._LoginTimerStop()
    QI(#SCGlogin).LoginFailed("Tote Server Unavailable")
    
  end if
  
  
end

on _SplitControlData(me, str)
  data = [:]
  
  oldDelim = the itemdelimiter
  the itemdelimiter = ","
  
  -- Separate 3 itemsPairs 
  -- ex :JSESSIONID=8c30fa5e18d62a2c2711,RESPONSE=0,MESSAGE=Valid Login
  repeat with i=1 to str.item.count
    itemPair = str.item[i]
    
    -- Seperate Name and Value
    the itemdelimiter = "="
    lName = itemPair.item[1]
    lValue = itemPair.item[2]
    the itemdelimiter = ","
    
    if lName = "JSESSIONID" then
      data[#JSESSIONID] = lValue
    else if lName = "RESPONSE" then
      data[#RESPONSE] = INTEGER(lValue)
    else if lName = "MESSAGE" then
      data[#MESSAGE] = lValue
    end if
    
  end repeat
  
  the itemdelimiter = oldDelim
  return data
  
end

--------------------------------------------------------------------
-- POLLING TIMER
--------------------------------------------------------------------

on _PollTimerStart (me)
  
  GenLog(#debug, "SCGCommunicator::PollTimerStart: Starting timer...")
  
  pPollTimer.ResetHit()
  pPollTimer.SetName("to,poll")
  pPollTimer.SetHitScale(pPollingPeriod)
  pPollTimer.SetTimeout(0)
  
  pPollTimer.SetHitCallback(#_PollTimerHitCallback, me)
  pPollTimer.SetTimeoutCallback(#_PollTimerTimeoutCallback, me)
  
  pPollTimer.Start()
  
end

on _PollTimerStop (me)
  pPollTimer.Stop()
  GenLog(#debug, "SCGCommunicator::Stop: PollTimer stopped!")
end

on _PollTimerHitCallback (me)
  me.Poll()
end

on _PollTimerTimeoutCallback (me)
  
end

--------------------------------------------------------------------
-- LOGIN TIMER
--------------------------------------------------------------------

on _LoginTimerStart (me)
  
  pLoginInProgress = true
  
  GenLog(#debug, "SCGCommunicator::LoginTimerStart: Starting timer...")
  
  pLoginTimer.ResetHit()
  pLoginTimer.SetName("to,poll")
  pLoginTimer.SetHitScale(1000)
  pLoginTimer.SetTimeout(pLoginTimeout)
  
  pLoginTimer.SetHitCallback(#_LoginTimerHitCallback, me)
  pLoginTimer.SetTimeoutCallback(#_LoginTimerTimeoutCallback, me)
  
  pLoginTimer.Start()
  
end

on _LoginTimerStop (me)
  pLoginInProgress = false
  pLoginTimer.Stop()
  GenLog(#debug, "SCGCommunicator::Stop: PollTimer stopped!")
end

on _LoginTimerHitCallback (me)
  QI(#loginTimeout).ResetHit()
end

on _LoginTimerTimeoutCallback (me)
  me.NetAbort()
  GenLog(#info, "SCGCommunicator::_LoginTimerTimeoutCallback::Login Connection Timed Out")
  QI(#SCGlogin).LoginFailed("Connection Timed Out")
  _LoginTimerStop()
end

