-------------------------------------------------------------------------------
-- Name: class XMLParser
-- Author: Jonathan Primeau
--
-- Description:
--     This class is high level API over the XML parser (xtra). It is easy 
--     to use. You only need to load an XML string and tell it to parse and it
--     will call the specified callback every times it encounters a tag.
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- P R O P E R T I E S
-------------------------------------------------------------------------------

property ancestor              -- Parent


-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = void
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  
  me.ancestor = void
  
  me.Stop()
end

on Start (me)
  lRunMode = QI(#status).GetRunMode()
  
  -- We don't want to use the security manager in authoring
  if (lRunMode = "Author") then return
  
  -- Activate the security only if required (configuration)
  if (QI(#config).GetSecurityActive() = 0) then return
  
  -- Not allowed to exit the projector
  _movie.exitlock = true
  
  lDeniedList = QI(#config).GetDeniedKeys() 
  lEscapeCode = QI(#config).GetSecurityPassphrase() 
  lLicense = QI(#config).GetSecurityLicense() 
  
  -- License verification
  lRetCode = UnlockKeyboardXtra(lLicense)
  if (lRetCode <> 0) then
    ShowMessage("Security License Failed")
    return()
  end if
  
  -- Get platform and admin privileges
  lWin = KeyboardControlGetWin32Platform()  
  lIsAdmin = KeyboardControlIsAdmin()
  
  -- Initializing the internal filter
  lResult = KeyboardControlInit()
  lErrorCode = lResult.getAt(1)
  
  if (lErrorCode <> 0) then
    case (lErrorCode) of
      9: -- Restart is required
        lMsg = "SECURITY MANAGER" & RETURN & "The system must be restarted, the application will "
        lMsg = lMsg & "quit now and you must reboot the kiosk." & RETURN
        _movie.exitlock = false
        ShowMessage(lMsg)
        _movie.halt()
        
      11: -- Final activation is required
        --        lMsg = "SECURITY MANAGER" & RETURN & "This message appears after initialization on " & RETURN
        --        lMsg = lMsg & "Windows NT/2000 only." & RETURN
        --        lMsg = lMsg & "It's required to provide the final activation." & RETURN
        --        lMsg = lMsg & "To do so press any key to continue."
        --        ShowMessage(lMsg)
        
      otherwise:  -- Something went wrong
        if (lWin = "NT" and lIsAdmin = 0) then
          -- Maybe we simply do not have the required privileges
          lMsg = "Please logon with administrative privileges."
        else
          -- Something serious has happend
          lMsg = "Initialization failed with error codes " & lResult & RETURN
          lMsg = lMsg & " Please refer to Error Codes section of the manual"
        end if
        _movie.exitlock = false
        ShowMessage(lMsg)
        _movie.halt()
        
    end case
  end if
  
  -- This needs to be done on NT (see Key Control documentation)
  if (lWin = "NT") then
    baSendKeys(" ")
  end if
  
  -- Now configuring the internal filter
  lResult = KeyboardControlSetHotKey(lDeniedList, "HotKeyHandler")
  
  -- Conform the escape code to what the xtra required
  lEscapePhrase = ""
  lCount = lEscapeCode.char.count
  repeat with i = 1 to lCount
    if (i = lCount) then
      lEscapePhrase = lEscapePhrase & lEscapeCode.Char[i]
    else
      lEscapePhrase = lEscapePhrase & lEscapeCode.Char[i] & "+"
    end if
  end repeat
  
  lResult = KeyboardControlSetStopSeq(lEscapePhrase, "StopSeqHandler")
  
  -- And finally starting
  lResult = KeyboardControlStart()
end

on AllowQuit (me)
  -- This handler is called whenever the stop sequence is typed
  _sound.beep()
  _movie.exitlock = false
end

on Stop (me)
  if (QI(#config).GetSecurityActive() = 0) then return
  
  -- Need to stop filtering and to uninitialize the internal filter
  lResult = KeyboardControlStop()
  
  -- Here we uninitialize the filter completely so this movie can be run on 
  -- NT/2000/XP only if you logged on as administrator. Call KeyboardControlUnInit()
  -- instead of KeyboardControlUnInitCompletely() if you want to run it with non-
  -- administrative privileges
  lResult = KeyboardControlUnInitCompletely()
end

on KeepFocus (me)
  -- Don't let other applications take the focus
  if (QI(#status).GetRunMode() <> "Author") then
    if (QI(#config).GetSecurityKeepFocus() = true) then
      if (baActiveWindow() <> baWinHandle()) then
        baWindowToFront(baWinHandle())
        baActivateWindow(baWinHandle())
      end if
    end if
  end if
end


-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  
end