-------------------------------------------------------------------------------
-- Name: class Terminal
-- Author: Jonathan Primeau
--
-- Description:
--     This is a bridge between Flash and Director for the Terminal.
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- P R O P E R T I E S
------------------------------------------------------------------------------- 

property ancestor              -- Parent
property pTerminal
property pIP                   -- Server IP
property pPort                 -- Server port
property pMessageCallback      -- Callback when getting a message
property pConnectionCallback   -- Callback when connection is made
property pIsConnected


-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = void
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  me.ancestor = void
end

on SetIP (me, theIP)
  me.pIP = theIP
end

on SetPort (me, thePort)
  me.pPort = thePort  
end

on SetConnectionCallback (me, theHandler, theObject)
  me.pConnectionCallback.handler = theHandler
  me.pConnectionCallback.object = theObject
end

on SetMessageCallback (me, theHandler, theObject)
  me.pMessageCallback.handler = theHandler
  me.pMessageCallback.object = theObject
end

on Connect (me)
  me.pIsConnected = false
  
  GenLog(#debug, "Terminal::Connect: Connecting to" && me.pIP & ":" & me.pPort & "...")
  me.pTerminal.connect(me.pIP, me.pPort)
end

on Disconnect (me)
  me.pIsConnected = false
  
  GenLog(#debug, "Terminal::Disconnect: Disconnecting...")
  me.pTerminal.close()
end

on IsConnected (me)
  return(me.pIsConnected)
end

on SendCommand (me, theObj)
  lStr = theObj
  if (me.pIsConnected) then
    GenLog(#info, "Terminal::SendCommand: Sending" && lStr & "...")
    me.pTerminal.send(theObj)
  else
    GenLog(#error, "Terminal::SendCommand: Not connected!")
  end if
end

on OnConnect (me, theStatus)
  if (value(theStatus) = true) then
    GenLog(#debug, "Terminal::OnConnect: OK connected to server!")
    me.pIsConnected = true
  else
    -- This is really bad...
    GenLog(#error, "Terminal::OnConnect: Could not connect to server!")
    QI(#status).SetStatus(#state, "Critical")
    SvrLog("Terminal: Cannot connect to terminal")
    
    me.pIsConnected = false
    
    if (QI(#init).IsInitialized() = true and QI(#init).IsLoaded() = true) then
      -- We need to reinitialize, we will be out of service
      QI(#fatalError).Show()
      -- Cannot send EndPatron because we are not connected to the server
      QI(#opsCom).Disconnect()
      QI(#init).ConnectionTimeout()
    end if
  end if
  if (me.pConnectionCallback.handler <> void and me.pConnectionCallback.object <> void) then
    call(me.pConnectionCallback.handler, me.pConnectionCallback.object, theStatus)
  end if
end

on OnData (me, theData)
  lData = SearchReplaceString(theData, "&quot;", QUOTE)
  GenLog(#info, "Terminal::OnData: Data:" && theData)
  if (me.pMessageCallback.handler <> void and me.pMessageCallback.object <> void) then
    call(me.pMessageCallback.handler, me.pMessageCallback.object, lData)
  end if
end

on OnClose (me)
  me.pIsConnected = false
  GenLog(#debug, "Terminal::OnClose: Disconnected!")
end


-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  lFlashConnector = sprite("Terminal")
  me.pTerminal = lFlashConnector.Terminal
  
  me.pIP = "127.0.0.1"
  me.pPort = 4634
  me.pMessageCallback = [#handler: void, #object: void]
  me.pConnectionCallback = [#handler: void, #object: void]
  me.pIsConnected = false
end
