-------------------------------------------------------------------------------
-- Name: class VideoList
-- Author: Jonathan Primeau
--
-- Description:
--     This class inherates from ItemList. It represents the Video channel
--     list.
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- P R O P E R T I E S
------------------------------------------------------------------------------- 

property ancestor              -- Parent

property pCameraList           -- Camera list received from server
property pCameraTitle          -- Camera title received from server

property pCurrCameraList       -- Camera list currently used
property pCurrCameraTitle      -- Camera title currently used

property pSelectedCamera
property pCameraCount

property pEventChannelList
property pVideoXML

property pCurrentWagerChannel
property pCurrentWagerEvent

property pSelectedID

property pAdTopFile
property pCurrAdTopFile


-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = script("class ItemList").new()
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  
  me.UnregisterListeners()
  
  me.ancestor = void
end

on RegisterListeners (me)
  QI(#events).RegisterListener(#ChannelChanged, #OnChannelChanged, me)
end

on UnregisterListeners (me)
  QI(#events).UnregisterListener(#ChannelChanged, #OnChannelChanged, me)  
end

on OnChannelChanged (me, theParams)
  lOldChannel = theParams.oldChannel
  lNewChannel = theParams.newChannel
  
  me.CheckWagerOnEvent(lNewChannel)
end

on CheckWagerOnEvent (me, theChannel)
  lSection = QI(#status).GetStatus().area
  -- Wagering is only available when not in tryout mode
  if (QI(#status).GetStatus().login <> "Tryout") then
    lCount = me.pEventChannelList.count
    repeat with i = 1 to lCount
      lEvent = me.pEventChannelList[i].event
      lChannel = me.pEventChannelList[i].channel
      if (theChannel = lChannel) then
        
        -- Move the wager sprite according to the section
        if (lSection = "video") then
          sprite("WagerOnThisEvent").locH = 170
          sprite("WagerOnThisEvent,text").locH = 47
        else if (lSection = "video_medium") then
          sprite("WagerOnThisEvent").locH = 830
          sprite("WagerOnThisEvent,text").locH = 706
        end if
        
        me.pCurrentWagerChannel = theChannel
        me.pCurrentWagerEvent = lEvent
        return()
      end if
    end repeat
  end if
  
  if (lSection = "video" or lSection = "video_medium") then
    -- Hide the sprite if no wagering channel
    sprite("WagerOnThisEvent").locH = -170
    sprite("WagerOnThisEvent,text").locH = -170
  end if
end

on DoWagerOnEvent (me)
  -- Wagering is only available when not in tryout mode
  if (QI(#status).GetStatus().login <> "Tryout") then
    --lTarget = "ogn://navigate/wagering" & lParam
    
    comInfo = QI(#config).GetSciGamesInfo()
    lURL = comInfo.getAprop(#wagerOnThisEventUrl)
    
    lParam = "?TrackCode=" & me.pCurrentWagerEvent
    lTarget = lURL & lParam
    QI(#nav).Navigate(lTarget, true)
  end if
end

on SetVideoXML (me, theXML)
  me.pVideoXML = theXML
end

on DisplayVideo (me)
  -- Reload and display the video section
  GenLog(#debug, "VideoList::DisplayVideo: Displaying video section...")  
  
  -- Get latest information
  me.UpdateData()
  
  -- We want the view all
  me.ResetTab()
  
  -- Get latest events
  me.LoadEventChannelList()
  me.CheckWagerOnEvent(QI(#wintv).GetChannel())
  
  -- Display
  me.DisplayTabs()
  
  lParams = QI(#nav).GetParams()
  lForceChannel = false
  if (lParams <> [:]) then
    if (lParams.getaProp(#channel) <> void) then
      me.SetChannel(integer(lParams.channel))
      lForceChannel = true
    end if
  end if
  
  if (lForceChannel = false) then
    -- Force the first line to be selected
    me.SetSelectedLine(1)
    
    -- Change channel will also display the page
    me.ChangeChannel()
  end if
  
  GenLog(#debug, "VideoList::DisplayVideo: video section displayed!")  
end

on DisplayVideoMedium (me)
  -- Reload and display the video section
  GenLog(#debug, "VideoList::DisplayVideoMedium: Displaying video_medium section...")  
  
  -- Use latest information
  me.UpdateData()
  
  me.ResetTab()
  
  -- Get latest events
  me.LoadEventChannelList()
  me.CheckWagerOnEvent(QI(#wintv).GetChannel())
  
  -- Get current channel information
  lIndex = me.GetSelectedIndex()
  lChannel = me.GetLineInfo(lIndex)
  lName = ""
  if (lChannel.getaProp(#name) <> void) then
    lName = lChannel.name
  end if
  
  -- Set the ad
  sprite("ad_top").member.filename = me.pCurrAdTopFile
  -- Center ad
  sprite("ad_top").rect = rect(652, 144, 1016, 257)
  
  -- Set the channel name
  sprite("channelName").member.text = lName
  
  GenLog(#debug, "VideoList::DisplayVideoMedium: video_medium section displayed!")  
end

on DisplayVideoLive (me)
  -- Reload and display the video section
  GenLog(#debug, "VideoList::DisplayVideoLive: Displaying video_live section...")  
  
  -- Use latest information
  me.pCurrCameraList = duplicate(me.pCameraList)
  me.pCurrCameraTitle = me.pCameraTitle
  
  me.DisplayCameraInfo()
  
  -- Force the first line to be selected
  me.SetSelectedCamera(1)
  
  -- Change channel
  me.ChangeLiveChannel()
  
  GenLog(#debug, "VideoList::DisplayVideoMedium: video_live section displayed!")  
end

on Load (me)
  GenLog(#debug, "VideoList::Load: Loading list...")
  
  if (me.pVideoXML = "") then
    GenLog(#error, "VideoList::Load: Empty XML!")
    SvrLog("Channel: Received an empty XML")
    return(-1)
  end if
  
  -- Clear old data
  ancestor.ClearList()
  me.pCameraList = []
  me.pCameraTitle = ""
  me.pAdTopFile = ""
  
  -- Load and parse XML
  me.LoadXMLFromString(me.pVideoXML)
  lStatus = me.Parse(#_XMLCallback)
  if (lStatus <> 0) then
    me.RevertData()
    return(lStatus)
  end if
  
  if (me.GetCategoryCount() < 2 or me.GetCategoryCount() > 8) then
    GenLog(#error, "VideoList::Load: The number of categories should between 2 and 8!")
    SvrLog("Channel: The number of categories should between 2 and 8")
    me.RevertData()
    return(-2)
  end if
  
  lMaxCamera = me.pCameraList.count
  if (lMaxCamera <> me.pCameraCount) then
    GenLog(#error, "VideoList::Load: The number of camera is not" && me.pCameraCount & "!")
    SvrLog("Channel: The number of camera is not" && me.pCameraCount)
    me.RevertData()
    return(-3)
  end if
  
  -- Check if the ad exists
  if (QI(#file).FileExists(me.pAdTopFile) = true) then
    me.pCurrAdTopFile = me.pAdTopFile
  else
    GenLog(#error, "VideoList::Load: The following ad doesn't exist:" && me.pAdTopFile)
    SvrLog("Channel: The following ad doesn't exist:" && me.pAdTopFile)
    me.RevertData()
    return(-4)
  end if
  
  GenLog(#debug, "VideoList::Load: List loaded!")
  return(0)
end

on DisplayCameraInfo (me)
  member("video_live,title").text = me.pCurrCameraTitle
  
  lMaxCamera = me.pCurrCameraList.count
  if (lMaxCamera > 6) then lMaxCamera = 6
  
  repeat with i = 1 to lMaxCamera
    member("video_live,camera,name," & i).text = me.pCurrCameraList[i].name
  end repeat
end

on LoadEventChannelList (me)
  if (me.pEventChannelList = []) then
    QI(#terminalCom).Events(#_EventChannelListCallback, me)
  end if
end

on GetEventChannelList (me)
  return(me.pEventChannelList)
end

on GetLineInfo (me, theLineIndex)
  lLineInfo = ancestor.GetLineInfo(theLineIndex)
  
  -- Take what we need from the line info
  lInfo = [:]
  
  if (lLineInfo.getaProp(#id) <> void) then
    lInfo.addProp(#id, lLineInfo.id)
  end if
  
  if (lLineInfo.getaProp(#name) <> void) then
    lInfo.addProp(#name, lLineInfo.name)
  end if
  
  if (lLineInfo.getaProp(#channel) <> void) then
    lInfo.addProp(#channel, lLineInfo.channel)
  end if
  
  return(lInfo)
end

on GetCameraInfo (me, theIndex)
  lMaxCamera = me.pCurrCameraList.count
  if (theIndex <= lMaxCamera) then
    return(me.pCurrCameraList[theIndex])
  end if
  return([:])
end

on SetSelectedLine (me, theLine)
  ancestor.SetSelectedLine(theLine)
  
  lInfo = me.GetLineInfo(theLine)
  if (lInfo <> [:]) then
    me.pSelectedID = integer(lInfo.id)
  end if
end

on IsSelected (me, theLine)
  lInfo = me.GetLineInfo(theLine)
  if (lInfo <> [:]) then
    lID = integer(lInfo.id)
    if (lID = me.pSelectedID) then
      return(true)
    end if
  end if
  return(false)
end

on SetChannel (me, theChannel)
  -- Is the channel in the list?
  lChannelID = me._GetChannelID(theChannel)
  if (lChannelID <> 0) then
    me.SetSelectedID(lChannelID)
    me.ChangeChannel()
  else
    QI(#wintv).SetChannel(theChannel)
    
    ActLog(#access, "Channel", theChannel)
    
    me._SetChannelLabel("")
    me.pSelectedID = 0
  end if
end

on ChangeChannel (me)
  lInfo = me.GetLineInfo(me.pSelectedLine)
  if (lInfo <> [:]) then
    lID = integer(lInfo.id)
    lName = string(lInfo.name)
    lChannel = integer(lInfo.channel)
    QI(#wintv).SetChannel(lChannel)
    
    me._SetChannelLabel(lName)
    
    ActLog(#access, "Channel", lName & "," && lChannel)
    ACTLOG(#navigation, "User" ,"ogn://navigate/video/"& lChannel& "," & lName)
    -- Keep the ID
    me.pSelectedID = me.GetSelectedID()
    
    -- Update the page
    lCurrentSection = QI(#status).GetStatus().area
    if (lCurrentSection = "video") then
      me.AdjustPage()
    end if
    
    
  end if
end

on ChannelUp (me)
  me.GoNextIndex()
  me.ChangeChannel()
end

on ChannelDown (me)
  me.GoPreviousIndex()
  me.ChangeChannel()
end

on SetSelectedCamera (me, theLine)
  me.pSelectedCamera = theLine
end

on IsCameraSelected (me, theLine)
  if (theLine = me.pSelectedCamera) then
    return(true)
  else
    return(false)
  end if
end

on ChangeLiveChannel (me)
  lInfo = me.GetCameraInfo(me.pSelectedCamera)
  if (lInfo <> [:]) then
    lName = string(lInfo.name)
    lChannel = integer(lInfo.channel)
    QI(#wintv).SetChannel(lChannel)
    
    me._SetChannelLabel(lName)
    me._SetLiveCamImage(me.pSelectedCamera)
  end if
end

on LiveChannelUp (me)
  if (me.pSelectedCamera >= me.pCameraCount) then
    me.pSelectedCamera = 1
  else
    me.pSelectedCamera = me.pSelectedCamera + 1
  end if
  
  me.ChangeLiveChannel()
end

on LiveChannelDown (me)
  if (me.pSelectedCamera <= 1) then
    me.pSelectedCamera = me.pCameraCount
  else
    me.pSelectedCamera = me.pSelectedCamera - 1
  end if
  
  me.ChangeLiveChannel()
end


-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  --me.Clear()
  
  me.pMaxCategory = 7
  me.pItemPerPage = 10
  
  me.pLineSpritePrefix = "video,channel,"
  me.pTabSpritePrefix = "video,categories_tab,"
  me.pListName = "video"
  
  me.pFieldList.add("name")
  me.pFieldList.add("description")
  
  me.pCameraList = []
  me.pCameraTitle = ""
  
  me.pCurrCameraList = []
  me.pCurrCameraTitle = ""
  
  me.pEventChannelList = []
  
  me.pSelectedID = 0
  
  me.pVideoXML = ""
  
  me.pSelectedCamera = 1
  me.pCameraCount = 6
  
  me.pAdTopFile = ""
  me.pCurrAdTopFile = ""
end

-------------------------------------------------------------------------------
-- void _XMLCallback (me, theItemName, theItemValue)
--
-- Callback used by the XML parser to return data. This method is the brain of
-- the XML processing because it knows what it has to look for. It is specified
-- to the XML Parse method and is called everytime it encounters an XML tag.
-- If you need to retrieve information from a new tag, you just have to add
-- the symbol corresponding and use the appropriate path. You can easily
-- retrieve any data you want using GetItemValue from the XML parser. Then you
-- have to add the retieved information to the appropriate list.
--
-- Parameters:
--     me (object): this pointer.
--     theItemName (symbol): The current item name being parsed.
--     theItemValue (list): Contains the information for the current item.
--
-------------------------------------------------------------------------------
on _XMLCallback (me, theItemName, theItemValue)
  case (theItemName) of
    #ad_definition:
      lAdTopFile = string(me.GetAttribute(theItemValue, "ad_top"))
      
      lAdFolder = the moviePath & "Ads\"
      
      if (lAdTopFile <> "" and lAdTopFile <> "None") then
        me.pAdTopFile = lAdFolder & lAdTopFile
      else
        me.pAdTopFile = lAdFolder & "empty.png"
      end if
      
    #video:
      -- Get site information
      lName = me.GetItemValue(theItemValue, "name")
      lDescription = me.GetItemValue(theItemValue, "description")
      lCategory = me.GetItemValue(theItemValue, "category")
      lChannel = me.GetItemValue(theItemValue, "channel")
      lEvent = me.GetItemValue(theItemValue, "event")
      if not voidp(lEvent) then
        pEventChannelList.append ([#channel: lChannel, #event: lEvent])
      end if
      
      
      lCategoryIndex = me._GetCategoryIndex(lCategory)
      if (lCategoryIndex = 0) then
        lCategoryIndex = me.AddCategory(lCategory)
      end if
      
      lCurrentChannel = [:]
      lCurrentChannel.addProp(#id, me.GetCount() + 1)
      lCurrentChannel.addProp(#name, lName)
      lCurrentChannel.addProp(#description, lDescription)
      lCurrentChannel.addProp(#channel, lChannel)
      
      me.AddItemToCategory(lCategoryIndex, lCurrentChannel)
      me.AddItemToAll(lCategoryIndex, lCurrentChannel)
      
    #live_event:
      lTitle = me.GetItemValue(theItemValue, "title")
      me.pCameraTitle = lTitle
      
    #camera:
      -- Get camera information
      lName = me.GetItemValue(theItemValue, "name")
      lChannel = me.GetItemValue(theItemValue, "channel")
      
      lCurrentCamera = [:]
      lCurrentCamera.addProp(#name, lName)
      lCurrentCamera.addProp(#channel, lChannel)
      
      me.pCameraList.add(lCurrentCamera)
  end case
end

on _EventChannelListCallback (me, theContent)
  me.LoadXMLFromString(theContent)
  me.Parse(#_EventChannelListXMLCallback)
end

on _EventChannelListXMLCallback (me, theItemName, theItemValue)
  case (theItemName) of
    #event:
      lChannel = integer(me.GetAttribute(theItemValue, "channel"))
      lEvent = integer(me.GetAttribute(theItemValue, "event"))
      if (lChannel <> void) then
        lCurrent = [:]
        lCurrent.addProp(#event, integer(lEvent))
        lCurrent.addProp(#channel, integer(lChannel))
        me.pEventChannelList.add(lCurrent)
      end if
  end case
end

on _GetChannelID (me, theChannel)
  return(me.GetItemID(#channel, theChannel))
end

on _SetChannelLabel (me, theName)
  sprite("channelName").member.text = theName
end

on _SetLiveCamImage (me, theCamera)
  sprite("trackMap").member = member("video_live,track_map_" & theCamera)
end